@@ -62,7 +62,7 @@ static gboolean connection_cleanup(gpointer ptr);
 /* We try to avoid too many connections per server */
 #define MAX_CONNECTIONS_PER_SERVER 20
 
-G_LOCK_DEFINE_STATIC(imap_servers);
+static pthread_mutex_t imap_servers_lock = PTHREAD_MUTEX_INITIALIZER;
 static GHashTable *imap_servers = NULL;
 
 struct handle_info {
@@ -113,26 +113,27 @@ static void libbalsa_imap_server_set_username(LibBalsaServer * server,
     if(server->host && name) { /* we have been initialized... */
         LibBalsaImapServer *imap_server = LIBBALSA_IMAP_SERVER(server);
         
-        G_LOCK(imap_servers);
+        pthread_mutex_lock(&imap_servers_lock);
         g_hash_table_steal(imap_servers, imap_server->key);
         g_free(imap_server->key);
         imap_server->key = g_strdup_printf("%s@%s", name, server->host);
         g_hash_table_insert(imap_servers, imap_server->key, imap_server);
-        G_UNLOCK(imap_servers);
+        pthread_mutex_unlock(&imap_servers_lock);
     }
     (parent_class)->set_username(server, name);
 }
-static void libbalsa_imap_server_set_host(LibBalsaServer * server,
-                      const gchar * host, gboolean use_ssl)
+static void
+libbalsa_imap_server_set_host(LibBalsaServer * server,
+                              const gchar * host, gboolean use_ssl)
 {
     if(server->user && host) { /* we have been initialized... */
         LibBalsaImapServer *imap_server = LIBBALSA_IMAP_SERVER(server);
-        G_LOCK(imap_servers);
+        pthread_mutex_lock(&imap_servers_lock);
         g_hash_table_steal(imap_servers, imap_server->key);
         g_free(imap_server->key);
         imap_server->key = g_strdup_printf("%s@%s", server->user, host);
         g_hash_table_insert(imap_servers, imap_server->key, imap_server);
-        G_UNLOCK(imap_servers);
+        pthread_mutex_unlock(&imap_servers_lock);
     }
     (parent_class)->set_host(server, host, use_ssl);
 }
@@ -152,7 +153,6 @@ libbalsa_imap_server_class_init(LibBalsaImapServerClass * klass)
     server_class->set_username = libbalsa_imap_server_set_username;
     server_class->set_host = libbalsa_imap_server_set_host;
 #if 0
-    klass->set_password = libbalsa_imap_server_real_set_password;
     klass->get_password = NULL; /* libbalsa_imap_server_real_get_password; */
 #endif
 }
@@ -188,9 +188,9 @@ libbalsa_imap_server_finalize(GObject * object)
     server = LIBBALSA_SERVER(object);
     imap_server = LIBBALSA_IMAP_SERVER(object);
 
-    G_LOCK(imap_servers);
+    pthread_mutex_lock(&imap_servers_lock);
     g_hash_table_remove(imap_servers, imap_server->key);
-    G_UNLOCK(imap_servers);
+    pthread_mutex_unlock(&imap_servers_lock);
     
     g_source_remove(imap_server->connection_cleanup_id);
 
@@ -297,6 +297,7 @@ lb_imap_server_info_new(LibBalsaServer *server)
     case LIBBALSA_TLS_REQUIRED: mode = IMAP_TLS_REQUIRED; break;
     }
     imap_handle_set_tls_mode(handle, mode);
+    imap_handle_set_option(handle, IMAP_OPT_ANONYMOUS, server->try_anonymous);
     return info;
 }
 
@@ -384,15 +385,15 @@ static LibBalsaImapServer* get_or_create(const gchar *username,
     gchar *key;
 
     if (!imap_servers) {
-        G_LOCK(imap_servers);
+        pthread_mutex_lock(&imap_servers_lock);
         if (!imap_servers)
             imap_servers = g_hash_table_new(g_str_hash, g_str_equal);
-        G_UNLOCK(imap_servers);
+        pthread_mutex_unlock(&imap_servers_lock);
     }
 
     /* lookup username@host */
     key = g_strdup_printf("%s@%s", username, host);
-    G_LOCK(imap_servers);
+    pthread_mutex_lock(&imap_servers_lock);
     imap_server = g_hash_table_lookup(imap_servers, key);
     if (!imap_server) {
         imap_server = g_object_new(LIBBALSA_TYPE_IMAP_SERVER, NULL);
@@ -402,7 +403,7 @@ static LibBalsaImapServer* get_or_create(const gchar *username,
         g_free(key);
         g_object_ref(imap_server);
     }
-    G_UNLOCK(imap_servers);
+    pthread_mutex_unlock(&imap_servers_lock);
     return imap_server;
 }
 
@@ -453,6 +454,8 @@ libbalsa_imap_server_new_from_config(void)
         server->user = tmp_server.user;
         server->host = tmp_server.host;
     }
+    d1 = libbalsa_conf_get_bool_with_default("Anonymous", &d);
+    if(!d) server->try_anonymous = !!d1;
     server->use_ssl |= libbalsa_conf_get_bool("SSL=false");
     tls_mode = libbalsa_conf_get_int_with_default("TLSMode", &d);
     if(!d) server->tls_mode = tls_mode;
@@ -499,17 +502,24 @@ handle_connection_error(int rc, struct handle_info *info,
                         LibBalsaServer *server, GError **err)
 {
     gchar *msg = imap_mbox_handle_get_last_msg(info->handle);
-    if(rc == IMAP_AUTH_FAILURE)
-        libbalsa_server_set_password(server, NULL);
-    if(rc != IMAP_CONNECT_FAILED) {
-        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
-                    LIBBALSA_MAILBOX_AUTH_ERROR,
-                    _("Cannot connect to the server: %s"), msg);
-    } else
+    switch(rc) {
+    case IMAP_AUTH_FAILURE:
+        libbalsa_server_set_password(server, NULL); break;
+    case IMAP_CONNECT_FAILED:
         g_set_error(err, LIBBALSA_MAILBOX_ERROR,
                     LIBBALSA_MAILBOX_NETWORK_ERROR,
                     _("Cannot connect to the server"));
-    
+        break;
+    case IMAP_AUTH_CANCELLED:
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                    LIBBALSA_MAILBOX_AUTH_CANCELLED,
+                    _("Cannot connect to the server: %s"), msg);
+        break;
+    default:
+        g_set_error(err, LIBBALSA_MAILBOX_ERROR,
+                    LIBBALSA_MAILBOX_AUTH_ERROR,
+                    _("Cannot connect to the server: %s"), msg);
+    }    
     g_free(msg);
     lb_imap_server_info_free(info);
 }
@@ -787,10 +797,10 @@ static void close_all_connections_cb(gpointer key, gpointer value,
 void
 libbalsa_imap_server_close_all_connections(void)
 {
-    G_LOCK(imap_servers);
-    if(imap_servers) 
+    pthread_mutex_lock(&imap_servers_lock);
+    if (imap_servers)
         g_hash_table_foreach(imap_servers, close_all_connections_cb, NULL);
-    G_UNLOCK(imap_servers);
+    pthread_mutex_unlock(&imap_servers_lock);
     libbalsa_imap_purge_temp_dir(0);
 }
 
