@@ -115,14 +115,7 @@ gchar *libbalsa_guess_ldap_server(void);
 
 gchar *libbalsa_guess_imap_inbox(void);
 
-gchar *libbalsa_guess_ldap_base(void);
-gchar *libbalsa_guess_ldap_name(void);
 
-gchar *libbalsa_guess_ldif_file(void);
-
-gboolean libbalsa_ldap_exists(const gchar *server);
-
-void libbalsa_assure_balsa_dir(void);
 gchar* libbalsa_date_to_utf8(const time_t *date, const gchar *date_string);
 LibBalsaMessageStatus libbalsa_get_icon_from_flags(LibBalsaMessageFlag flags);
 
@@ -138,11 +131,58 @@ pthread_t libbalsa_get_main_thread(void);
 gboolean libbalsa_am_i_subthread(void);
 void libbalsa_threads_init(void);
 void libbalsa_threads_destroy(void);
+gboolean libbalsa_threads_has_lock(void);
 #else
 #define libbalsa_am_i_subthread() FALSE
+#define libbalsa_threads_has_lock() TRUE
 #endif /* BALSA_USE_THREADS */
 void libbalsa_message(const char *fmt, ...);
 gchar * libbalsa_rot(const gchar * pass);
 
+typedef enum {
+    LIBBALSA_PROGRESS_NO = 0,
+    LIBBALSA_PROGRESS_YES
+} LibBalsaProgress;
+#define LIBBALSA_PROGRESS_INIT LIBBALSA_PROGRESS_NO
+/* We will not use the progress bar if the number of increments is less
+ * than LIBBALSA_PROGRESS_MIN_COUNT, and we will not update the fraction
+ * if it has increased by less than LIBBALSA_PROGRESS_MIN_UPDATE. */
+#define LIBBALSA_PROGRESS_MIN_COUNT  400
+#define LIBBALSA_PROGRESS_MIN_UPDATE 0.02
+
+extern void (*libbalsa_progress_set_text) (LibBalsaProgress * progress,
+                                           const gchar * text,
+                                           guint total);
+extern void (*libbalsa_progress_set_fraction) (LibBalsaProgress * progress,
+                                               gdouble fraction);
+
+/*
+ * Face and X-Face header support.
+ */
+gchar *libbalsa_get_header_from_path(const gchar * header,
+                                     const gchar * path, gsize * size,
+                                     GError ** err);
+GtkWidget *libbalsa_get_image_from_face_header(const gchar * content,
+                                               GError ** err);
+#if HAVE_COMPFACE
+GtkWidget *libbalsa_get_image_from_x_face_header(const gchar * content,
+                                                 GError ** err);
+#endif                          /* HAVE_COMPFACE */
+
+GQuark libbalsa_image_error_quark(void);
+#define LIBBALSA_IMAGE_ERROR libbalsa_image_error_quark()
+enum LibBalsaImageError {
+    LIBBALSA_IMAGE_ERROR_NO_DATA
+#if HAVE_COMPFACE
+        ,
+    LIBBALSA_IMAGE_ERROR_FORMAT,
+    LIBBALSA_IMAGE_ERROR_BUFFER,
+    LIBBALSA_IMAGE_ERROR_BAD_DATA
+#endif                          /* HAVE_COMPFACE */
+};
+
+#if HAVE_GTKSOURCEVIEW
+GtkWidget *libbalsa_source_view_new(gboolean highlight_phrases, GdkColor *q_colour);
+#endif                          /* HAVE_GTKSOURCEVIEW */
 
 #endif                          /* __LIBBALSA_H__ */
