@@ -121,6 +121,12 @@ typedef enum {
     LB_MAILBOX_SHOW_TO
 } LibBalsaMailboxShow;
 
+typedef enum {
+    LB_MAILBOX_SUBSCRIBE_NO,
+    LB_MAILBOX_SUBSCRIBE_YES,
+    LB_MAILBOX_SUBSCRIBE_UNSET
+} LibBalsaMailboxSubscribe;
+
 typedef enum {
     LB_FETCH_RFC822_HEADERS = 1<<0, /* prepare all rfc822 headers */
     LB_FETCH_STRUCTURE      = 1<<1  /* prepare message structure */
@@ -167,11 +173,12 @@ struct _LibBalsaMailboxView {
     LibBalsaMailboxSortType      sort_type;
     LibBalsaMailboxSortFields    sort_field;
     LibBalsaMailboxShow          show;
-    unsigned exposed:1;
-    unsigned open:1;
-    unsigned in_sync:1;		/* view is in sync with config */
-    unsigned frozen:1;		/* don't update view if set    */
-    unsigned used:1;		/* keep track of usage         */
+    LibBalsaMailboxSubscribe     subscribe;
+    gboolean exposed;
+    gboolean open;
+    gboolean in_sync;		/* view is in sync with config */
+    gboolean frozen;		/* don't update view if set    */
+    gboolean used;		/* keep track of usage         */
 
 #ifdef HAVE_GPGME
     LibBalsaChkCryptoMode gpg_chk_mode;
@@ -213,10 +220,12 @@ struct _LibBalsaMailbox {
     GPtrArray *mindex;  /* the basic message index used for index
                          * displaying/columns of GtkTreeModel interface
                          * and NOTHING else. */
-    GNode *msg_tree; /* the possibly filtered tree of messages */
+    GNode *msg_tree; /* the possibly filtered tree of messages;
+                      * gdk lock MUST BE HELD when accessing. */
     LibBalsaCondition *view_filter; /* to choose a subset of messages
                                      * to be displayed, e.g., only
                                      * undeleted. */
+    LibBalsaCondition *persistent_view_filter;
 
     /* info fields */
     gboolean has_unread_messages;
@@ -233,6 +242,9 @@ struct _LibBalsaMailbox {
 
     /* Whether to reassemble a message from its parts. */
     gboolean no_reassemble;
+
+    /* Whether the tree has been changed since some event. */
+    gboolean msg_tree_changed;
 };
 
 /* Search iter */
@@ -469,9 +481,12 @@ void libbalsa_mailbox_index_entry_set_no(LibBalsaMailboxIndexEntry *entry,
 void libbalsa_mailbox_index_entry_free(LibBalsaMailboxIndexEntry *entry);
 void libbalsa_mailbox_index_set_flags(LibBalsaMailbox *mailbox,
 				      unsigned msgno, LibBalsaMessageFlag f);
-void libbalsa_mailbox_set_view_filter(LibBalsaMailbox   *mailbox,
-                                      LibBalsaCondition *filter_condition,
-                                      gboolean update_immediately);
+gboolean libbalsa_mailbox_set_view_filter(LibBalsaMailbox * mailbox,
+                                          LibBalsaCondition *
+                                          filter_condition,
+                                          gboolean update_immediately);
+void libbalsa_mailbox_make_view_filter_persistent(LibBalsaMailbox *
+                                                  mailbox);
 
 gboolean libbalsa_mailbox_can_do(LibBalsaMailbox *mailbox,
                                  enum LibBalsaMailboxCapability cap);
@@ -506,6 +521,9 @@ void libbalsa_mailbox_set_sort_field(LibBalsaMailbox * mailbox,
 				     LibBalsaMailboxSortFields sort_field);
 gboolean libbalsa_mailbox_set_show(LibBalsaMailbox * mailbox,
 				   LibBalsaMailboxShow show);
+gboolean libbalsa_mailbox_set_subscribe(LibBalsaMailbox * mailbox,
+                                        LibBalsaMailboxSubscribe
+                                        subscribe);
 void libbalsa_mailbox_set_exposed(LibBalsaMailbox * mailbox,
 				  gboolean exposed);
 void libbalsa_mailbox_set_open(LibBalsaMailbox * mailbox, gboolean open);
@@ -529,6 +547,8 @@ LibBalsaMailboxSortType libbalsa_mailbox_get_sort_type(LibBalsaMailbox *
 LibBalsaMailboxSortFields libbalsa_mailbox_get_sort_field(LibBalsaMailbox *
 							  mailbox);
 LibBalsaMailboxShow libbalsa_mailbox_get_show(LibBalsaMailbox * mailbox);
+LibBalsaMailboxSubscribe libbalsa_mailbox_get_subscribe(LibBalsaMailbox *
+                                                        mailbox);
 gboolean libbalsa_mailbox_get_exposed(LibBalsaMailbox * mailbox);
 gboolean libbalsa_mailbox_get_open(LibBalsaMailbox * mailbox);
 gint libbalsa_mailbox_get_filter(LibBalsaMailbox * mailbox);
