@@ -241,37 +241,46 @@ libbalsa_mailbox_local_remove_files(LibBalsaMailboxLocal * local)
 static gboolean message_match_real(LibBalsaMailbox * mailbox, guint msgno,
                                    LibBalsaCondition * cond);
 static LibBalsaMessage *
-libbalsa_mailbox_local_load_message(LibBalsaMailbox * mailbox,
-                                    GNode **sibling, guint msgno)
+libbalsa_mailbox_local_load_message(LibBalsaMailboxLocal * local,
+                                    GNode ** sibling, guint msgno)
 {
     LibBalsaMessage *msg = NULL;
     LibBalsaMessageFlag flags;
-    LibBalsaMailbox *mbx = LIBBALSA_MAILBOX(mailbox);
-    flags = 
-	LIBBALSA_MAILBOX_LOCAL_GET_CLASS(mailbox)->load_message(mailbox,
-								msgno,
-                                                                &msg);
+    LibBalsaMailbox *mbx = LIBBALSA_MAILBOX(local);
+    gboolean match;
 
-    if ( (flags & LIBBALSA_MESSAGE_FLAG_NEW) ) {
-	if(!(flags & LIBBALSA_MESSAGE_FLAG_DELETED))
-            mbx->unread_messages++;
-        if(mbx->first_unread == 0)
+    flags =
+        LIBBALSA_MAILBOX_LOCAL_GET_CLASS(local)->load_message(local, msgno,
+                                                              &msg);
+
+    if ((flags & LIBBALSA_MESSAGE_FLAG_NEW)
+        && !(flags & LIBBALSA_MESSAGE_FLAG_DELETED)) {
+        mbx->unread_messages++;
+        if (mbx->first_unread == 0)
             mbx->first_unread = msgno;
     }
 
-    if(msg) {
+    if (flags & LIBBALSA_MESSAGE_FLAG_RECENT) {
         gchar *id;
-        msg->msgno = msgno;
-        msg->mailbox = mbx;
+
+        if (!msg)
+            msg = libbalsa_mailbox_get_message(mbx, msgno);
+
         if (libbalsa_message_is_partial(msg, &id)) {
             libbalsa_mailbox_try_reassemble(mbx, id);
             g_free(id);
         }
     }
 
-    if (!mbx->view_filter
-        || message_match_real(mbx, msgno, mbx->view_filter))
-        libbalsa_mailbox_msgno_inserted(mbx, msgno, mbx->msg_tree, sibling);
+    if (!mbx->view_filter)
+        match = TRUE;
+    else if (!libbalsa_condition_is_flag_only(mbx->view_filter,
+                                              mbx, msgno, &match))
+        match = message_match_real(mbx, msgno, mbx->view_filter);
+
+    if (match)
+        libbalsa_mailbox_msgno_inserted(mbx, msgno, mbx->msg_tree,
+                                        sibling);
 
     return msg;
 }
@@ -451,8 +460,9 @@ lbm_local_save_tree(LibBalsaMailboxLocal * local)
     gchar *template;
 #endif                          /* GLIB_CHECK_VERSION(2, 8, 0) */
 
-    if (!mailbox->msg_tree)
+    if (!mailbox->msg_tree || !mailbox->msg_tree_changed)
         return;
+    mailbox->msg_tree_changed = FALSE;
 
     filename = lbm_local_get_cache_filename(local);
 
@@ -523,6 +533,7 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
 {
     LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
     gchar *filename;
+    gchar *name;
     gchar *contents;
     gsize length;
     GError *err = NULL;
@@ -531,22 +542,31 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
     guint8 *seen;
 
     filename = lbm_local_get_cache_filename(local);
+    name = mailbox->name ? g_strdup(mailbox->name) :
+        g_path_get_basename(libbalsa_mailbox_local_get_path(local));
 
     if (!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
         /* No error, but we return FALSE so the caller can grab all the
          * message info needed to rethread from scratch. */
+        if (libbalsa_mailbox_total_messages(mailbox) > 0)
+            libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
+                                 _("Cache file for mailbox %s "
+                                   "will be created"), name);
         g_free(filename);
+        g_free(name);
         return FALSE;
     }
 
     if (!g_file_get_contents(filename, &contents, &length, &err)) {
         libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                             _("Failed to read cache file \"%s\": %s"),
+                             _("Failed to read cache file %s: %s"),
                              filename, err->message);
         g_error_free(err);
         g_free(filename);
+        g_free(name);
         return FALSE;
     }
+    g_free(filename);
 
     info = (LibBalsaMailboxLocalTreeInfo *) contents;
     /* Sanity checks: first the file should have >= 1 record. */
@@ -554,14 +574,18 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
         /* First record is (0, total): */
         || info->msgno != 0
         /* Total must be > 0 (no file is created for empty tree). */
-        || (*total = info->value.total) == 0
-        || *total > libbalsa_mailbox_total_messages(mailbox)) {
-        libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                             _("Bad cache file \"%s\""), filename);
+        || info->value.total == 0
+        || info->value.total > libbalsa_mailbox_total_messages(mailbox)) {
+        libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
+                             _("Cache file for mailbox %s "
+                               "will be repaired"), name);
         g_free(contents);
-        g_free(filename);
+        g_free(name);
         return FALSE;
     }
+    *total = info->value.total;
+
+    gdk_threads_enter();
 
     seen = g_new0(guint8, *total);
     parent = mailbox->msg_tree;
@@ -569,11 +593,13 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
     while (++info < (LibBalsaMailboxLocalTreeInfo *) (contents + length)) {
         if (info->msgno == 0 || info->msgno > *total
             || seen[info->msgno - 1]) {
-            libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                                 _("Bad cache file \"%s\""), filename);
+            libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
+                                 _("Cache file for mailbox %s "
+                                   "will be repaired"), name);
             g_free(seen);
             g_free(contents);
-            g_free(filename);
+            g_free(name);
+            gdk_threads_leave();
             return FALSE;
         }
         seen[info->msgno - 1] = TRUE;
@@ -591,12 +617,13 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
                 parent = parent->parent;
                 if (!parent) {
                     /* We got to the root without finding the parent. */
-                    libbalsa_information(LIBBALSA_INFORMATION_WARNING,
-                                         _("Bad cache file \"%s\""),
-                                         filename);
+                    libbalsa_information(LIBBALSA_INFORMATION_DEBUG,
+                                         _("Cache file for mailbox %s "
+                                           "will be repaired"), name);
                     g_free(seen);
                     g_free(contents);
-                    g_free(filename);
+                    g_free(name);
+    		    gdk_threads_leave();
                     return FALSE;
                 }
             }
@@ -606,12 +633,19 @@ lbm_local_restore_tree(LibBalsaMailboxLocal * local, guint * total)
         if (libbalsa_mailbox_msgno_has_flags(mailbox, info->msgno,
                                              LIBBALSA_MESSAGE_FLAG_NEW,
                                              LIBBALSA_MESSAGE_FLAG_DELETED))
+        {
             ++mailbox->unread_messages;
+            if (mailbox->first_unread == 0 ||
+                mailbox->first_unread > info->msgno)
+                mailbox->first_unread = info->msgno;
+        }
     }
 
     g_free(seen);
     g_free(contents);
-    g_free(filename);
+    g_free(name);
+
+    gdk_threads_leave();
 
     return TRUE;
 }
@@ -648,7 +682,8 @@ lbm_local_save_tree_idle(LibBalsaMailboxLocal * local)
 static void
 lbm_local_queue_save_tree(LibBalsaMailboxLocal * local)
 {
-    if (!local->save_tree_id) {
+    if (LIBBALSA_MAILBOX(local)->msg_tree_changed
+        && !local->save_tree_id) {
         g_object_ref(local);
         local->save_tree_id =
             g_idle_add((GSourceFunc) lbm_local_save_tree_idle, local);
@@ -670,16 +705,25 @@ libbalsa_mailbox_local_close_mailbox(LibBalsaMailbox * mailbox,
         local->sync_id = 0;
     }
 
+    /* Restore the persistent view before saving the tree. */
+    libbalsa_mailbox_set_view_filter(mailbox,
+                                     mailbox->persistent_view_filter, TRUE);
+
     if (local->thread_id) {
+        /* Rethread immediately. */
+        LibBalsaMailboxThreadingType cur_type =
+            libbalsa_mailbox_get_threading_type(mailbox);
         g_source_remove(local->thread_id);
         local->thread_id = 0;
+        libbalsa_mailbox_set_threading(mailbox, cur_type);
     }
 
     if (local->save_tree_id) {
+        /* Save immediately. */
         g_source_remove(local->save_tree_id);
         local->save_tree_id = 0;
+        lbm_local_save_tree(local);
     }
-    lbm_local_save_tree(local);
 
     if (local->threading_info) {
 	/* Free the memory owned by local->threading_info, but neither
@@ -725,6 +769,7 @@ message_match_real(LibBalsaMailbox *mailbox, guint msgno,
      * fetch it here, and that will also populate entry and info. */
     if (!entry || !info) {
         message = libbalsa_mailbox_get_message(mailbox, msgno);
+        libbalsa_mailbox_local_cache_message(local, msgno, message);
         entry = g_ptr_array_index(mailbox->mindex, msgno-1);
         info  = g_ptr_array_index(local->threading_info, msgno - 1);
     }
@@ -854,9 +899,10 @@ libbalsa_mailbox_local_message_match(LibBalsaMailbox * mailbox,
  * loads incrementally new messages, if any.
  *  Mailbox lock MUST BE HELD before calling this function.
  */
-static gboolean
-lbm_local_cache_message(LibBalsaMailboxLocal * local, guint msgno,
-                        LibBalsaMessage * message)
+void
+libbalsa_mailbox_local_cache_message(LibBalsaMailboxLocal * local,
+                                     guint msgno,
+                                     LibBalsaMessage * message)
 {
     gpointer *entry;
     LibBalsaMailboxLocalInfo *info;
@@ -865,14 +911,14 @@ lbm_local_cache_message(LibBalsaMailboxLocal * local, guint msgno,
                                    message);
 
     if (!local->threading_info)
-        return FALSE;
+        return;
 
     while (local->threading_info->len < msgno)
         g_ptr_array_add(local->threading_info, NULL);
     entry = &g_ptr_array_index(local->threading_info, msgno - 1);
 
     if (*entry)
-        return FALSE;
+        return;
 
     *entry = info = g_new(LibBalsaMailboxLocalInfo, 1);
     info->message_id = g_strdup(message->message_id);
@@ -880,8 +926,6 @@ lbm_local_cache_message(LibBalsaMailboxLocal * local, guint msgno,
         libbalsa_message_refs_for_threading(message);
     info->sender =
         internet_address_list_to_string(message->headers->from, FALSE);
-
-    return TRUE;
 }
 
 void
@@ -894,9 +938,14 @@ libbalsa_mailbox_local_load_messages(LibBalsaMailbox *mailbox,
     GNode *lastn;
 
     g_return_if_fail(LIBBALSA_IS_MAILBOX_LOCAL(mailbox));
-    if (!mailbox->msg_tree)
+
+    gdk_threads_enter();
+
+    if (!mailbox->msg_tree) {
 	/* Mailbox is closed, or no view has been created. */
+        gdk_threads_leave();
 	return;
+    }
 
     local = (LibBalsaMailboxLocal *) mailbox;
     lastno = libbalsa_mailbox_total_messages(mailbox);
@@ -904,13 +953,15 @@ libbalsa_mailbox_local_load_messages(LibBalsaMailbox *mailbox,
     lastn = g_node_last_child(mailbox->msg_tree);
     while (++msgno <= lastno){
 	LibBalsaMessage *msg =
-	    libbalsa_mailbox_local_load_message(mailbox, &lastn, msgno);
+	    libbalsa_mailbox_local_load_message(local, &lastn, msgno);
 	if (msg) {
-            lbm_local_cache_message(local, msgno, msg);
+            libbalsa_mailbox_local_cache_message(local, msgno, msg);
             g_object_unref(msg);
         }
     }
 
+    gdk_threads_leave();
+
     if (new_messages) {
 	libbalsa_mailbox_run_filters_on_reception(mailbox);
 	libbalsa_mailbox_set_unread_messages_flag(mailbox,
@@ -919,51 +970,6 @@ libbalsa_mailbox_local_load_messages(LibBalsaMailbox *mailbox,
     }
 }
 
-static gboolean
-lbm_local_thread_idle(LibBalsaMailboxLocal * local)
-{
-    LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
-
-    libbalsa_lock_mailbox(mailbox);
-    gdk_threads_enter();
-
-    if (MAILBOX_OPEN(mailbox)) {
-        LibBalsaMailboxThreadingType cur_type =
-            libbalsa_mailbox_get_threading_type(mailbox);
-
-        libbalsa_mailbox_set_threading(mailbox, cur_type);
-    }
-    local->thread_id = 0;
-    g_object_unref(local);
-
-    gdk_threads_leave();
-    libbalsa_unlock_mailbox(mailbox);
-
-    return FALSE;
-}
-
-void
-libbalsa_mailbox_local_cache_message(LibBalsaMailboxLocal * local,
-                                     guint msgno,
-                                     LibBalsaMessage * message)
-{
-    LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
-    gboolean natural =
-        (libbalsa_mailbox_get_threading_type(mailbox) ==
-         LB_MAILBOX_THREADING_FLAT
-         && libbalsa_mailbox_get_sort_field(mailbox) ==
-         LB_MAILBOX_SORT_NO);
-
-    if (lbm_local_cache_message(local, msgno, message)
-            /* Message was not previously cached... */
-        && !natural     /* ...and we need to check threading and sorting */
-        && !local->thread_id) {
-        g_object_ref(local);
-        local->thread_id =
-            g_idle_add((GSourceFunc) lbm_local_thread_idle, local);
-    }
-}
-
 /*
  * Threading
  */
@@ -987,33 +993,41 @@ libbalsa_mailbox_local_set_threading(LibBalsaMailbox * mailbox,
     LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mailbox);
 
     libbalsa_mailbox_local_set_threading_info(local);
+#if defined(DEBUG_LOADING_AND_THREADING)
     printf("before load_messages: time=%lu\n", (unsigned long) time(NULL));
+#endif
     if (!mailbox->msg_tree) {   /* first reference */
         guint total = 0;
         gboolean natural = (thread_type == LB_MAILBOX_THREADING_FLAT
                             && libbalsa_mailbox_get_sort_field(mailbox) ==
                             LB_MAILBOX_SORT_NO);
 
-        mailbox->msg_tree = g_node_new(NULL);
+        libbalsa_mailbox_set_msg_tree(mailbox, g_node_new(NULL));
         if (!lbm_local_restore_tree(local, &total)) {
             /* Bad or no cache file: start over. */
-            g_node_destroy(mailbox->msg_tree);
-            mailbox->msg_tree = g_node_new(NULL);
+            libbalsa_mailbox_set_msg_tree(mailbox, g_node_new(NULL));
             total = 0;
-            if (!natural)
-                /* Get all message info, so we can thread and sort from
-                 * scratch. */
-                libbalsa_mailbox_prepare_threading(mailbox, NULL, 0);
         }
-        /* If the mailbox has more messages than we restored into the
-         * tree, we assume that the remainder are new: */
-        libbalsa_mailbox_local_load_messages(mailbox, total);
+        mailbox->msg_tree_changed = FALSE;
+
+        if (total < libbalsa_mailbox_total_messages(mailbox)) {
+            if (!natural)
+                /* Get message info for all messages that weren't restored,
+                 * so we can thread and sort them correctly before the
+                 * mailbox is displayed. */
+                libbalsa_mailbox_prepare_threading(mailbox, NULL, total);
+            libbalsa_mailbox_local_load_messages(mailbox, total);
+        }
+
+#if defined(DEBUG_LOADING_AND_THREADING)
         printf("after load messages: time=%lu\n", (unsigned long) time(NULL));
+#endif
         if (natural)
             /* No need to thread. */
             return;
     }
 
+    gdk_threads_enter();
     switch (thread_type) {
     case LB_MAILBOX_THREADING_JWZ:
         lbml_threading_jwz(mailbox);
@@ -1023,7 +1037,10 @@ libbalsa_mailbox_local_set_threading(LibBalsaMailbox * mailbox,
         lbml_threading_simple(mailbox, thread_type);
         break;
     }
+    gdk_threads_leave();
+#if defined(DEBUG_LOADING_AND_THREADING)
     printf("after threading time=%lu\n", (unsigned long) time(NULL));
+#endif
 
     lbm_local_queue_save_tree(local);
 }
@@ -1047,21 +1064,34 @@ libbalsa_mailbox_local_msgno_removed(LibBalsaMailbox * mailbox,
 
 static void
 lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
-                             LibBalsaCondition *view_filter)
+                             LibBalsaCondition * view_filter)
 {
+    guint total;
+    LibBalsaProgress progress = LIBBALSA_PROGRESS_INIT;
     LibBalsaMailboxSearchIter *iter_view;
     guint msgno;
 
+    total = libbalsa_mailbox_total_messages(mailbox);
+    if (view_filter
+        && !libbalsa_condition_is_flag_only(view_filter, NULL, 0, NULL)) {
+        gchar *text;
+
+        text = g_strdup_printf(_("Filtering %s"), mailbox->name);
+        libbalsa_progress_set_text(&progress, text, total);
+        g_free(text);
+    }
+
     iter_view = libbalsa_mailbox_search_iter_new(view_filter);
-    for (msgno = 1;
-	 msgno <= libbalsa_mailbox_total_messages(mailbox);
-	 msgno++)
-	libbalsa_mailbox_msgno_filt_check(mailbox, msgno, iter_view, FALSE);
+    for (msgno = 1; msgno <= total; msgno++) {
+        libbalsa_mailbox_msgno_filt_check(mailbox, msgno, iter_view,
+                                          FALSE);
+        libbalsa_progress_set_fraction(&progress, ((gdouble) msgno) /
+                                       ((gdouble) total));
+    }
+    libbalsa_progress_set_text(&progress, NULL, 0);
     libbalsa_mailbox_search_iter_free(iter_view);
 
-    if(mailbox->view_filter)
-        libbalsa_condition_free(mailbox->view_filter);
-    mailbox->view_filter = view_filter;
+    lbm_local_queue_save_tree(LIBBALSA_MAILBOX_LOCAL(mailbox));
 }
 
 /*
@@ -1070,35 +1100,99 @@ lbm_local_update_view_filter(LibBalsaMailbox * mailbox,
  * at LibBalsaMailbox.
  */
 
-/* Helper. */
-static void
+/* Helper: returns TRUE if msgno was not already cached, which means we
+ * have new data for sorting or threading. */
+static gboolean
 lbm_local_prepare_msgno(LibBalsaMailboxLocal * local, guint msgno)
 {
+    LibBalsaMessage *message;
+
     if (msgno <= local->threading_info->len
         && g_ptr_array_index(local->threading_info, msgno - 1))
-        return;
+        return FALSE;
 
-    g_object_unref(libbalsa_mailbox_get_message((LibBalsaMailbox *) local,
-                                                 msgno));
+    message =
+        libbalsa_mailbox_get_message((LibBalsaMailbox *) local, msgno);
+    libbalsa_mailbox_local_cache_message(local, msgno, message);
+    g_object_unref(message);
+
+    return TRUE;
 }
 
-/* The class method. */
+/* Idle handler. */
+static gboolean
+lbm_local_thread_idle(LibBalsaMailboxLocal * local)
+{
+    LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
+
+    libbalsa_lock_mailbox(mailbox);
+
+    if (MAILBOX_OPEN(mailbox)) {
+        LibBalsaMailboxThreadingType cur_type =
+            libbalsa_mailbox_get_threading_type(mailbox);
+
+        libbalsa_mailbox_set_threading(mailbox, cur_type);
+    }
+    local->thread_id = 0;
+
+    libbalsa_unlock_mailbox(mailbox);
+    g_object_unref(local);
+
+    return FALSE;
+}
+
+/* The class method; prepare messages in the list, or if msgnos == NULL,
+ * from len + 1 to the end of the mailbox. */
 static void
 libbalsa_mailbox_local_prepare_threading(LibBalsaMailbox * mailbox,
                                          guint * msgnos, guint len)
 {
     LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mailbox);
     guint msgno;
+    gboolean need_thread = FALSE;
 
     libbalsa_mailbox_local_set_threading_info(local);
 
-    if (msgnos && len)
-        do
-            lbm_local_prepare_msgno(local, *msgnos++);
-        while (--len);
-    else
-        for (msgno = local->threading_info->len; msgno > 0; --msgno)
-            lbm_local_prepare_msgno(local, msgno);
+    if (msgnos) {
+        while (len) {
+            if (lbm_local_prepare_msgno(local, *msgnos++))
+                need_thread = TRUE;
+            --len;
+        }
+    } else {
+        gchar *text;
+        guint total;
+        LibBalsaProgress progress = LIBBALSA_PROGRESS_INIT;
+
+        text = g_strdup_printf(_("Preparing %s"), mailbox->name);
+        total = libbalsa_mailbox_total_messages(mailbox);
+        libbalsa_progress_set_text(&progress, text, total - len);
+        g_free(text);
+
+        for (msgno = len + 1; msgno <= total; msgno++) {
+            if (lbm_local_prepare_msgno(local, msgno)) {
+                need_thread = TRUE;
+                libbalsa_progress_set_fraction(&progress,
+                                               ((gdouble) msgno) /
+                                               ((gdouble) (total - len)));
+            }
+        }
+
+        libbalsa_progress_set_text(&progress, NULL, 0);
+    }
+
+    if (need_thread && !local->thread_id) {
+        LibBalsaMailbox *mailbox = LIBBALSA_MAILBOX(local);
+
+        if (libbalsa_mailbox_get_threading_type(mailbox) !=
+            LB_MAILBOX_THREADING_FLAT
+            || libbalsa_mailbox_get_sort_field(mailbox) !=
+            LB_MAILBOX_SORT_NO) {
+            g_object_ref(local);
+            local->thread_id =
+                g_idle_add((GSourceFunc) lbm_local_thread_idle, local);
+        }
+    }
 }
 
 /* fetch message structure method: all local mailboxes have their own
@@ -1201,7 +1295,7 @@ static void
 lbml_info_setup(LibBalsaMailbox * mailbox, ThreadingInfo * ti)
 {
     ti->mailbox = mailbox;
-    ti->root = g_node_new(NULL);
+    ti->root = g_node_new(mailbox->msg_tree);
     ti->id_table = g_hash_table_new(g_str_hash, g_str_equal);
     ti->subject_table = NULL;
     ti->unthreaded =
@@ -1706,7 +1800,7 @@ lbml_construct(GNode * node, ThreadingInfo * ti)
 {
     GNode *msg_node;
 
-    if ((msg_node = node->data)) {
+    if (node->parent && (msg_node = node->data)) {
         GNode *msg_parent = node->parent->data;
 
         if (msg_parent && msg_node->parent != msg_parent
@@ -1942,6 +2036,7 @@ libbalsa_mailbox_local_duplicate_msgnos(LibBalsaMailbox * mailbox)
     table = g_hash_table_new(g_str_hash, g_str_equal);
     msgnos = g_array_new(FALSE, FALSE, sizeof(guint));
 
+    /* We need all the message-ids. */
     libbalsa_mailbox_prepare_threading(mailbox, NULL, 0);
 
     for (i = 0; i < local->threading_info->len; i++) {
