@@ -109,12 +109,13 @@ libbalsa_mailbox_maildir_total_messages(LibBalsaMailbox * mailbox);
 
 /* MailboxLocal class methods */
 static gint lbm_maildir_check_files(const gchar * path, gboolean create);
-static void lbm_maildir_set_path(LibBalsaMailboxLocal * mailbox,
+static void lbm_maildir_set_path(LibBalsaMailboxLocal * local,
                                  const gchar * path);
-static void lbm_maildir_remove_files(LibBalsaMailboxLocal *mailbox);
-static LibBalsaMessageFlag lbm_maildir_load_message(LibBalsaMailbox * mailbox,
-						     guint msgno, 
-                                                     LibBalsaMessage **msg);
+static void lbm_maildir_remove_files(LibBalsaMailboxLocal * local);
+static LibBalsaMessageFlag lbm_maildir_load_message(LibBalsaMailboxLocal *
+                                                    local, guint msgno,
+                                                    LibBalsaMessage **
+                                                    msg);
 static guint lbm_maildir_fileno(LibBalsaMailboxLocal * local, guint msgno);
 
 /* util functions */
@@ -647,9 +648,6 @@ libbalsa_mailbox_maildir_close_mailbox(LibBalsaMailbox * mailbox,
     LibBalsaMailboxMaildir *mdir = LIBBALSA_MAILBOX_MAILDIR(mailbox);
     guint len;
 
-    g_assert(mdir->msgno_2_msg_info != NULL);
-    g_assert(mdir->messages_info != NULL);
-
     len = mdir->msgno_2_msg_info->len;
     libbalsa_mailbox_maildir_sync(mailbox, expunge);
     if (mdir->msgno_2_msg_info->len != len)
@@ -897,18 +895,15 @@ lbm_maildir_get_message(LibBalsaMailbox * mailbox, guint msgno)
     message->msgno = msgno;
     libbalsa_message_load_envelope(message);
 
-    libbalsa_mailbox_local_cache_message(LIBBALSA_MAILBOX_LOCAL(mailbox),
-                                         msgno, message);
-
     return message;
 }
 
 static LibBalsaMessageFlag
-lbm_maildir_load_message(LibBalsaMailbox * mailbox, guint msgno,
+lbm_maildir_load_message(LibBalsaMailboxLocal * local, guint msgno,
                          LibBalsaMessage ** msg)
 {
     struct message_info *msg_info =
-        message_info_from_msgno((LibBalsaMailboxMaildir *) mailbox, msgno);
+        message_info_from_msgno(LIBBALSA_MAILBOX_MAILDIR(local), msgno);
 
     if (msg_info->message)
         *msg = g_object_ref(msg_info->message);
