@@ -300,7 +300,8 @@ void libbalsa_message_body_unref(LibBalsaMessage * message);
 gboolean libbalsa_message_postpone(LibBalsaMessage * message,
 				   LibBalsaMailbox * draftbox,
 				   LibBalsaMessage * reply_message,
-				   gchar * fcc, gboolean flow);
+				   gchar ** extra_headers,
+				   gboolean flow);
 
 /*
  * misc message releated functions
