@@ -28,8 +28,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-#include <libgnome/libgnome.h>
-
 #include "libbalsa.h"
 #include "libbalsa_private.h"
 #include "rfc3156.h"
@@ -107,7 +105,7 @@ libbalsa_check_crypto_engine(gpgme_protocol_t protocol)
 	} else
 	    g_string_append_printf(message,
 				   _
-				   ("%s: could not retreive crypto engine information: %s."),
+				   ("%s: could not retrieve crypto engine information: %s."),
 				   gpgme_strsource(err),
 				   gpgme_strerror(err));
 	g_string_append_printf(message,
@@ -1128,10 +1126,12 @@ libbalsa_signature_info_to_gchar(GMimeGpgmeSigstat * info,
                attrs = g_string_append_c(attrs, ',');
            attrs = g_string_append(attrs, _(" invalid"));
        }
-       if (count > 1)
-           g_string_append_printf(msg, _("\nSubkey attributes:%s"), attrs->str);
-       else
-           g_string_append_printf(msg, _("\nSubkey attribute:%s"), attrs->str);
+       /* ngettext: string begins with a single space, so no space after
+        * the colon is correct punctuation (in English). */
+       g_string_append_printf(msg, ngettext("\nSubkey attribute:%s",
+                                            "\nSubkey attributes:%s",
+                                            count),
+                              attrs->str);
        g_string_free(attrs, TRUE);
     }
     if (info->issuer_name) {
@@ -1239,7 +1239,7 @@ check_gpg_child(gpointer data)
 	dialog =
 	    gtk_message_dialog_new(spawned_gpg->parent,
 				   GTK_DIALOG_DESTROY_WITH_PARENT,
-				   GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE,
+				   GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
 				   _
 				   ("Running gpg failed with return value %d:\n%s"),
 				   WEXITSTATUS(status), gpg_message);
