@@ -34,6 +34,7 @@
 
 #include "smtp-server.h"
 #include "libbalsa-conf.h"
+#include "misc.h"
 #include "i18n.h"
 
 static LibBalsaServerClass *parent_class = NULL;
@@ -70,7 +71,9 @@ libbalsa_smtp_server_finalize(GObject * object)
 
     auth_destroy_context(smtp_server->authctx);
     g_free(smtp_server->name);
+#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
     g_free(smtp_server->cert_passphrase);
+#endif                          /* HAVE_SMTP_TLS_CLIENT_CERTIFICATE */
 
     G_OBJECT_CLASS(parent_class)->finalize(object);
 }
@@ -254,6 +257,7 @@ libbalsa_smtp_server_get_name(LibBalsaSmtpServer * smtp_server)
     return smtp_server ? smtp_server->name : _("Default");
 }
 
+#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
 void
 libbalsa_smtp_server_set_cert_passphrase(LibBalsaSmtpServer * smtp_server,
                                          const gchar * passphrase)
@@ -267,6 +271,7 @@ libbalsa_smtp_server_get_cert_passphrase(LibBalsaSmtpServer * smtp_server)
 {
     return smtp_server->cert_passphrase;
 }
+#endif                          /* HAVE_SMTP_TLS_CLIENT_CERTIFICATE */
 
 auth_context_t
 libbalsa_smtp_server_get_authctx(LibBalsaSmtpServer * smtp_server)
@@ -281,6 +286,33 @@ libbalsa_smtp_server_get_big_message(LibBalsaSmtpServer * smtp_server)
     return smtp_server->big_message * 1024;
 }
 
+static gint
+smtp_server_compare(gconstpointer a, gconstpointer b)
+{
+    const LibBalsaSmtpServer *smtp_server_a = a;
+    const LibBalsaSmtpServer *smtp_server_b = b;
+
+    if (smtp_server_a->name && smtp_server_b->name)
+        return strcmp(smtp_server_a->name, smtp_server_b->name);
+
+    return smtp_server_a->name - smtp_server_b->name;
+}
+
+void
+libbalsa_smtp_server_add_to_list(LibBalsaSmtpServer * smtp_server,
+                                 GSList ** server_list)
+{
+    GSList *list;
+
+    if ((list =
+         g_slist_find_custom(*server_list, smtp_server,
+                             smtp_server_compare)) != NULL) {
+        g_object_unref(list->data);
+        *server_list = g_slist_delete_link(*server_list, list);
+    }
+
+    *server_list = g_slist_prepend(*server_list, smtp_server);
+}
 
 /* SMTP server dialog */
 
@@ -295,8 +327,8 @@ struct smtp_server_dialog_info {
     GtkWidget *host;
     GtkWidget *user;
     GtkWidget *pass;
-    GtkWidget *tlsm;
 #if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
+    GtkWidget *tlsm;
     GtkWidget *cert;
 #endif                          /* HAVE_SMTP_TLS_CLIENT_CERTIFICATE */
     GtkWidget *split_button;
@@ -326,9 +358,7 @@ static void
 smtp_server_add_widget(GtkWidget * table, gint row, const gchar * text,
                        GtkWidget * widget)
 {
-    GtkWidget *label = gtk_label_new_with_mnemonic(text);
-    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
-    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, row, row + 1);
+    GtkWidget *label = libbalsa_create_label(text, table, row);
     gtk_table_attach_defaults(GTK_TABLE(table), widget,
                               1, 2, row, row + 1);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), widget);
@@ -400,6 +430,7 @@ smtp_server_response(GtkDialog * dialog, gint response,
         libbalsa_server_set_password(server,
                                      gtk_entry_get_text(GTK_ENTRY
                                                         (sdi->pass)));
+#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
         switch (gtk_combo_box_get_active(GTK_COMBO_BOX(sdi->tlsm))) {
         case 0:
             server->tls_mode = Starttls_DISABLED;
@@ -413,7 +444,6 @@ smtp_server_response(GtkDialog * dialog, gint response,
         default:
             break;
         }
-#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
         libbalsa_smtp_server_set_cert_passphrase(sdi->smtp_server,
                                                  gtk_entry_get_text
                                                  (GTK_ENTRY(sdi->cert)));
@@ -513,36 +543,34 @@ libbalsa_smtp_server_dialog(LibBalsaSmtpServer * smtp_server,
     gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog), GTK_RESPONSE_OK,
                                       FALSE);
 
-#define HIG_PADDING 6
-    table = gtk_table_new(6, 2, FALSE);
-    gtk_table_set_row_spacings(GTK_TABLE(table), HIG_PADDING);
-    gtk_table_set_col_spacings(GTK_TABLE(table), HIG_PADDING);
+#define HIG_PADDING 12
+    table = libbalsa_create_table(6, 2);
     gtk_container_set_border_width(GTK_CONTAINER(table), HIG_PADDING);
     gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);
 
     row = 0;
-    smtp_server_add_widget(table, row, _("_Descriptive Name"),
+    smtp_server_add_widget(table, row, _("_Descriptive Name:"),
                            sdi->name = gtk_entry_new());
     if (smtp_server->name)
         gtk_entry_set_text(GTK_ENTRY(sdi->name), smtp_server->name);
     g_signal_connect(sdi->name, "changed", G_CALLBACK(smtp_server_changed),
                      sdi);
 
-    smtp_server_add_widget(table, ++row, _("_Server"),
+    smtp_server_add_widget(table, ++row, _("_Server:"),
                            sdi->host = gtk_entry_new());
     if (server->host)
         gtk_entry_set_text(GTK_ENTRY(sdi->host), server->host);
     g_signal_connect(sdi->host, "changed", G_CALLBACK(smtp_server_changed),
                      sdi);
 
-    smtp_server_add_widget(table, ++row, _("_User Name"),
+    smtp_server_add_widget(table, ++row, _("_User Name:"),
                            sdi->user = gtk_entry_new());
     if (server->user)
         gtk_entry_set_text(GTK_ENTRY(sdi->user), server->user);
     g_signal_connect(sdi->user, "changed", G_CALLBACK(smtp_server_changed),
                      sdi);
 
-    smtp_server_add_widget(table, ++row, _("_Pass Phrase"),
+    smtp_server_add_widget(table, ++row, _("_Pass Phrase:"),
                            sdi->pass = gtk_entry_new());
     gtk_entry_set_visibility(GTK_ENTRY(sdi->pass), FALSE);
     if (server->passwd)
@@ -550,13 +578,13 @@ libbalsa_smtp_server_dialog(LibBalsaSmtpServer * smtp_server,
     g_signal_connect(sdi->pass, "changed", G_CALLBACK(smtp_server_changed),
                      sdi);
 
-    smtp_server_add_widget(table, ++row, _("Use _TLS"), sdi->tlsm =
+#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
+    smtp_server_add_widget(table, ++row, _("Use _TLS:"), sdi->tlsm =
                            smtp_server_tls_widget(smtp_server));
     g_signal_connect(sdi->tlsm, "changed", G_CALLBACK(smtp_server_changed),
                      sdi);
 
-#if HAVE_SMTP_TLS_CLIENT_CERTIFICATE
-    smtp_server_add_widget(table, ++row, _("C_ertificate Pass Phrase"),
+    smtp_server_add_widget(table, ++row, _("C_ertificate Pass Phrase:"),
                            sdi->cert = gtk_entry_new());
     gtk_entry_set_visibility(GTK_ENTRY(sdi->cert), FALSE);
     if (smtp_server->cert_passphrase)
