@@ -58,6 +58,8 @@ auth_context_t libbalsa_smtp_server_get_authctx(LibBalsaSmtpServer *
                                                 smtp_server);
 guint libbalsa_smtp_server_get_big_message(LibBalsaSmtpServer *
                                            smtp_server);
+void libbalsa_smtp_server_add_to_list(LibBalsaSmtpServer * smtp_server,
+                                      GSList ** server_list);
 
 typedef void (*LibBalsaSmtpServerUpdate) (LibBalsaSmtpServer * smtp_server,
                                           GtkResponseType response,
