@@ -104,7 +104,7 @@ ask_password_real(LibBalsaServer * server, LibBalsaMailbox * mbox)
         server->remember_passwd = 
             !!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rememb));
         libbalsa_server_set_password(server, passwd);
-        if(old_rem != server->remember_passwd)
+        if( server->remember_passwd || old_rem )
             libbalsa_server_config_changed(server);
     }
     gtk_widget_destroy(dialog);
@@ -375,7 +375,7 @@ balsa_app_init(void)
 
     /* spell check */
 #if HAVE_GTKSPELL
-    balsa_app.spell_check = FALSE;
+    balsa_app.spell_check_lang = NULL;
 #else                           /* HAVE_GTKSPELL */
     balsa_app.module = SPELL_CHECK_MODULE_ASPELL;
     balsa_app.suggestion_mode = SPELL_CHECK_SUGGEST_NORMAL;
@@ -742,9 +742,11 @@ balsa_find_iter_by_data_func(GtkTreeModel * model, GtkTreePath * path,
 			       GtkTreeIter * iter, gpointer user_data)
 {
     struct balsa_find_iter_by_data_info *bf = user_data;
-    BalsaMailboxNode *mbnode;
+    BalsaMailboxNode *mbnode = NULL;
 
     gtk_tree_model_get(model, iter, 0, &mbnode, -1);
+    if(!mbnode)
+        return FALSE;
     if (mbnode == bf->data || mbnode->mailbox == bf->data) {
 	*bf->iter = *iter;
 	bf->found = TRUE;
@@ -833,76 +835,6 @@ balsa_remove_children_mailbox_nodes(BalsaMailboxNode * mbnode)
     g_slist_free(specials);
 }
 
-/* create_label:
-   Create a label and add it to a table in the first column of given row,
-   setting the keyval to found accelerator value, that can be later used 
-   in create_entry.
-*/
-GtkWidget *
-create_label(const gchar * label, GtkWidget * table, gint row)
-{
-    GtkWidget *w = gtk_label_new_with_mnemonic(label);
-
-    gtk_misc_set_alignment(GTK_MISC(w), 1.0, 0.5);
-    gtk_table_attach(GTK_TABLE(table), w, 0, 1, row, row + 1,
-		     GTK_FILL, GTK_FILL, 5, 5);
-    gtk_widget_show(w);
-    return w;
-}
-
-/* create_check:
-   creates a checkbox with a given label and places them in given array.
-*/
-GtkWidget *
-create_check(GtkDialog *mcw, const gchar *label, GtkWidget *table, gint row,
-             gboolean initval)
-{
-    GtkWidget *cb, *l;
-    
-    cb = gtk_check_button_new();
-
-    l = gtk_label_new_with_mnemonic(label);
-    gtk_label_set_mnemonic_widget(GTK_LABEL(l), cb);
-    gtk_misc_set_alignment(GTK_MISC(l), 0.0, 0.5);
-    gtk_widget_show(l);
-
-    gtk_container_add(GTK_CONTAINER(cb), l);
-
-    gtk_table_attach(GTK_TABLE(table), cb, 1, 2, row, row+1,
-		     GTK_FILL, GTK_FILL, 5, 5);
-
-    if(initval) 	
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cb), TRUE);
-
-    return cb;
-}
-
-/* Create a text entry and add it to the table */
-GtkWidget *
-create_entry(GtkDialog *mcw, GtkWidget * table, 
-	     GtkSignalFunc changed_func, gpointer data, gint row, 
-	     const gchar * initval, GtkWidget* hotlabel)
-{
-    GtkWidget *entry = gtk_entry_new();
-    gtk_table_attach(GTK_TABLE(table), entry, 1, 2, row, row + 1,
-		     GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 5);
-    if (initval) {
-        gint zero = 0;
-        
-        gtk_editable_insert_text(GTK_EDITABLE(entry), initval, -1, &zero);
-    }
-
-    gtk_label_set_mnemonic_widget(GTK_LABEL(hotlabel), entry);
-
-    /* Watch for changes... */
-    if(changed_func)
-	g_signal_connect(G_OBJECT(entry), "changed", 
-			 G_CALLBACK(changed_func), data);
-
-    gtk_widget_show(entry);
-    return entry;
-}
-
 /* balsa_find_index_by_mailbox:
    returns BalsaIndex displaying passed mailbox, or NULL, if mailbox is 
    not displayed.
@@ -920,7 +852,7 @@ balsa_find_index_by_mailbox(LibBalsaMailbox * mailbox)
 	  gtk_notebook_get_nth_page(GTK_NOTEBOOK(balsa_app.notebook), i));
 	 i++) {
         index = gtk_bin_get_child(GTK_BIN(page));
-	if (BALSA_INDEX(index)->mailbox_node != NULL
+	if (index && BALSA_INDEX(index)->mailbox_node
             && BALSA_INDEX(index)->mailbox_node->mailbox == mailbox)
 	    return BALSA_INDEX(index);
     }
