@@ -121,11 +121,11 @@ static GtkTargetEntry index_drag_types[] = {
 };
 
 static void bndx_drag_cb(GtkWidget* widget,
-                                GdkDragContext* drag_context,
-                                GtkSelectionData* data,
-                                guint info,
-                                guint time,
-                                gpointer user_data);
+                         GdkDragContext* drag_context,
+                         GtkSelectionData* data,
+                         guint info,
+                         guint time,
+                         gpointer user_data);
 
 /* Popup menu */
 static GtkWidget* bndx_popup_menu_create(BalsaIndex * index);
@@ -241,12 +241,9 @@ bndx_destroy(GtkObject * obj)
 						 G_SIGNAL_MATCH_DATA,
 						 0, 0, NULL, NULL, index);
 	    gtk_tree_view_set_model(GTK_TREE_VIEW(index), NULL);
-            /* Undo any temporary filters. */
-            libbalsa_mailbox_set_view_filter(mailbox,
-                                             balsa_window_get_view_filter
-                                             (balsa_app.main_window, TRUE),
-                                             TRUE);
+            gdk_threads_leave();
 	    libbalsa_mailbox_close(mailbox, balsa_app.expunge_on_close);
+            gdk_threads_enter();
 
 	    libbalsa_mailbox_search_iter_free(index->search_iter);
 	    index->search_iter = NULL;
@@ -895,44 +892,78 @@ balsa_index_new(void)
 }
 
 /**
- * balsa_index_scroll_on_open()
- * moves to the first unread message in the index, or the
- * last message if none is unread, and selects it.
+ * balsa_index_scroll_on_open() moves to the first unread message in
+ * the index, or the last message if none is unread, and selects
+ * it. Since this routine is usually called from a thread, we have to
+ * take care and and make sure the row is selected from the main
+ * thread only.
  */
+struct view_on_open_data {
+    BalsaIndex *bindex;
+    GtkTreePath *path;
+};
+static gboolean
+bi_view_on_open(struct view_on_open_data *data)
+{
+    gdk_threads_enter();
+    bndx_select_row(data->bindex, data->path);
+    gtk_tree_path_free(data->path);
+    g_object_unref(data->bindex);
+    g_free(data);
+    gdk_threads_leave();
+    return FALSE;
+}
+
 void
 balsa_index_scroll_on_open(BalsaIndex *index)
 {
     LibBalsaMailbox *mailbox = index->mailbox_node->mailbox;
     GtkTreeIter iter;
     GtkTreePath *path = NULL;
-    unsigned msgno;
+    gpointer view_on_open;
 
     balsa_index_update_tree(index, balsa_app.expand_tree);
     if (mailbox->first_unread) {
-	msgno = mailbox->first_unread;
+	unsigned msgno = mailbox->first_unread;
 	mailbox->first_unread = 0;
-    } else
-	msgno = libbalsa_mailbox_total_messages(mailbox);
-    if(msgno>0 &&
-       libbalsa_mailbox_msgno_find(mailbox, msgno, &path, &iter)) {
-        bndx_expand_to_row(index, path);
-	bndx_scroll_to_row(index, path);
-        if(balsa_app.view_message_on_open)
-            bndx_select_row(index, path);
-        gtk_tree_path_free(path);
+        if(!libbalsa_mailbox_msgno_find(mailbox, msgno, &path, &iter))
+            return; /* Oops! */
+    } else {
+        /* we want to scroll to the last one in current order. The
+           alternative which is to scroll to the most recently
+           delivered does not feel natural when other sorting order is
+           used */
+        int total = gtk_tree_model_iter_n_children
+            (GTK_TREE_MODEL(mailbox), NULL);
+        if(total == 0)
+            return;
+        gtk_tree_model_iter_nth_child (GTK_TREE_MODEL(mailbox), &iter, NULL,
+                                       total - 1);
+        path = gtk_tree_model_get_path(GTK_TREE_MODEL(mailbox), &iter);
     }
+
+    bndx_expand_to_row(index, path);
+    /* Scroll now, not in the idle handler, to make sure the initial
+     * view is correct. */
+    gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(index), path, NULL,
+                                 FALSE, 0, 0);
+
+    view_on_open =
+        g_object_get_data(G_OBJECT(mailbox), BALSA_INDEX_VIEW_ON_OPEN);
+    g_object_set_data(G_OBJECT(mailbox),
+                      BALSA_INDEX_VIEW_ON_OPEN,
+                      GINT_TO_POINTER(FALSE));
+    if ((view_on_open && GPOINTER_TO_INT(view_on_open))
+        || balsa_app.view_message_on_open) {
+        struct view_on_open_data *data = g_new(struct view_on_open_data,1);
+        data->bindex = index;
+        data->path = path;
+        g_object_ref(data->bindex);
+        g_idle_add((GSourceFunc)bi_view_on_open, data);
+    } else gtk_tree_path_free(path);
 }
 
-static LibBalsaCondition cond_undeleted =
-{
-    TRUE,
-    CONDITION_FLAG,
-    {
-       {
-           LIBBALSA_MESSAGE_FLAG_DELETED
-       }
-    }
-};
+static LibBalsaCondition *cond_undeleted;
 
 /* Callback for the mailbox's "row-inserted" signal; queue an idle
  * handler to expand the thread. */
@@ -1006,6 +1037,9 @@ bndx_mailbox_row_inserted_cb(LibBalsaMailbox * mailbox, GtkTreePath * path,
 
 /* balsa_index_load_mailbox_node:
    open mailbox_node, the opening is done in thread to keep UI alive.
+
+   Called NOT holding the gdk lock, so we must wrap gtk calls in
+   gdk_threads_{enter,leave}.
 */
 
 gboolean
@@ -1017,6 +1051,7 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
     gchar *msg;
     gboolean successp;
     gint try_cnt;
+    LibBalsaCondition *view_filter;
 
     g_return_val_if_fail(BALSA_IS_INDEX(index), TRUE);
     g_return_val_if_fail(index->mailbox_node == NULL, TRUE);
@@ -1024,16 +1059,18 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
     g_return_val_if_fail(LIBBALSA_IS_MAILBOX(mbnode->mailbox), TRUE);
 
     mailbox = mbnode->mailbox;
+
     msg = g_strdup_printf(_("Opening mailbox %s. Please wait..."),
 			  mbnode->mailbox->name);
+    gdk_threads_enter();
     gnome_appbar_push(balsa_app.appbar, msg);
+    gdk_threads_leave();
     g_free(msg);
+
     try_cnt = 0;
     do {
         g_clear_error(err);
-        gdk_threads_leave();
         successp = libbalsa_mailbox_open(mailbox, err);
-        gdk_threads_enter();
         if (!balsa_app.main_window)
             return FALSE;
 
@@ -1042,7 +1079,10 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
             break;
         balsa_mblist_close_lru_peer_mbx(balsa_app.mblist, mailbox);
     } while(try_cnt++<3);
+
+    gdk_threads_enter();
     gnome_appbar_pop(balsa_app.appbar);
+    gdk_threads_leave();
 
     if (!successp)
 	return TRUE;
@@ -1057,6 +1097,7 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
     /*
      * rename "from" column to "to" for outgoing mail
      */
+    gdk_threads_enter();
     tree_view = GTK_TREE_VIEW(index);
     if (libbalsa_mailbox_get_show(mailbox) == LB_MAILBOX_SHOW_TO) {
         GtkTreeViewColumn *column =
@@ -1072,21 +1113,30 @@ balsa_index_load_mailbox_node (BalsaIndex * index,
 	    	     G_CALLBACK(bndx_mailbox_row_inserted_cb), index);
 
     balsa_window_enable_mailbox_menus(balsa_app.main_window, index);
-    libbalsa_mailbox_set_view_filter(mailbox,
-                                     balsa_window_get_view_filter
-                                     (balsa_app.main_window, TRUE), FALSE);
+    gdk_threads_leave();
+
+    view_filter = balsa_window_get_view_filter(balsa_app.main_window, TRUE);
+    libbalsa_mailbox_set_view_filter(mailbox, view_filter, FALSE);
+    libbalsa_condition_unref(view_filter);
+    libbalsa_mailbox_make_view_filter_persistent(mailbox);
     libbalsa_mailbox_set_threading(mailbox,
                                    libbalsa_mailbox_get_threading_type
                                    (mailbox));
 
+    gdk_threads_enter();
     /* Set the tree store*/
 #ifndef GTK2_FETCHES_ONLY_VISIBLE_CELLS
     g_object_set_data(G_OBJECT(mailbox), "tree-view", tree_view);
 #endif
     gtk_tree_view_set_model(tree_view, GTK_TREE_MODEL(mailbox));
+    gdk_threads_leave();
 
     /* Create a search-iter for SEARCH UNDELETED. */
-    index->search_iter = libbalsa_mailbox_search_iter_new(&cond_undeleted);
+    if (!cond_undeleted)
+        cond_undeleted =
+            libbalsa_condition_new_flag_enum(TRUE,
+                                             LIBBALSA_MESSAGE_FLAG_DELETED);
+    index->search_iter = libbalsa_mailbox_search_iter_new(cond_undeleted);
     /* Note when this mailbox was opened, for use in auto-closing. */
     time(&index->mailbox_node->last_use);
 
@@ -1286,41 +1336,48 @@ balsa_index_find(BalsaIndex * index,
 				 BNDX_SEARCH_WRAP_NO));
 }
 
-static void
+static gboolean
 bndx_select_next_with_flag(BalsaIndex * index, LibBalsaMessageFlag flag)
 {
-    LibBalsaCondition cond_flag, cond_and;
+    LibBalsaCondition *cond_flag, *cond_and;
     LibBalsaMailboxSearchIter *search_iter;
+    gboolean retval;
 
-    g_return_if_fail(BALSA_IS_INDEX(index));
+    g_assert(BALSA_IS_INDEX(index));
 
-    cond_flag.negate      = FALSE;
-    cond_flag.type        = CONDITION_FLAG;
-    cond_flag.match.flags = flag;
-    cond_and.negate            = FALSE;
-    cond_and.type              = CONDITION_AND;
-    cond_and.match.andor.left  = &cond_flag;
-    cond_and.match.andor.right = &cond_undeleted;
-    search_iter = libbalsa_mailbox_search_iter_new(&cond_and);
+    cond_flag = libbalsa_condition_new_flag_enum(FALSE, flag);
+    cond_and =
+        libbalsa_condition_new_bool_ptr(FALSE, CONDITION_AND, cond_flag,
+                                        cond_undeleted);
+    libbalsa_condition_unref(cond_flag);
 
-    bndx_search_iter_and_select(index, search_iter,
-				BNDX_SEARCH_DIRECTION_NEXT,
-				BNDX_SEARCH_VIEWABLE_ANY,
-				BNDX_SEARCH_START_ANY,
-				BNDX_SEARCH_WRAP_YES);
+    search_iter = libbalsa_mailbox_search_iter_new(cond_and);
+    libbalsa_condition_unref(cond_and);
+
+    retval = bndx_search_iter_and_select(index, search_iter,
+                                         BNDX_SEARCH_DIRECTION_NEXT,
+                                         BNDX_SEARCH_VIEWABLE_ANY,
+                                         BNDX_SEARCH_START_ANY,
+                                         BNDX_SEARCH_WRAP_YES);
 
     libbalsa_mailbox_search_iter_free(search_iter);
+
+    return retval;
 }
 
-void
+gboolean
 balsa_index_select_next_unread(BalsaIndex * index)
 {
-    bndx_select_next_with_flag(index, LIBBALSA_MESSAGE_FLAG_NEW);
+    g_return_val_if_fail(BALSA_IS_INDEX(index), FALSE);
+
+    return bndx_select_next_with_flag(index, LIBBALSA_MESSAGE_FLAG_NEW);
 }
 
 void
 balsa_index_select_next_flagged(BalsaIndex * index)
 {
+    g_return_if_fail(BALSA_IS_INDEX(index));
+
     bndx_select_next_with_flag(index, LIBBALSA_MESSAGE_FLAG_FLAGGED);
 }
 
@@ -1329,6 +1386,8 @@ balsa_index_select(BalsaIndex * index, LibBalsaMessage * message)
 {
     GtkTreePath *path;
 
+    g_return_if_fail(BALSA_IS_INDEX(index));
+
     if (bndx_find_message(index, &path, NULL, message)) {
         bndx_select_row(index, path);
         gtk_tree_path_free(path);
@@ -1431,7 +1490,6 @@ bndx_mailbox_changed_func(BalsaIndex * bindex)
     bndx_changed_find_row(bindex);
 }
 
-
 /* bndx_mailbox_changed_cb:
    may be called from a thread. Use idle callback to update the view.
 */
@@ -1439,26 +1497,36 @@ struct index_info {
     BalsaIndex * bindex;
 };
 
+#define BNDX_IDLE_ARG "balsa-index-arg-key"
 static gboolean
 bndx_mailbox_changed_idle(struct index_info* arg)
 {
     gdk_threads_enter();
+
     if(arg->bindex) {
         g_object_remove_weak_pointer(G_OBJECT(arg->bindex),
                                      (gpointer) &arg->bindex);
 	bndx_mailbox_changed_func(arg->bindex);
+        g_object_set_data(G_OBJECT(arg->bindex), BNDX_IDLE_ARG, NULL);
     }
-    gdk_threads_leave();
     g_free(arg);
+
+    gdk_threads_leave();
     return FALSE;
 }
 
 static void
 bndx_mailbox_changed_cb(BalsaIndex * bindex)
 {
-    struct index_info *arg = g_new(struct index_info,1);
-    arg->bindex  = bindex;
+    struct index_info *arg;
+
+    if (g_object_get_data(G_OBJECT(bindex), BNDX_IDLE_ARG))
+        return;
+
+    arg = g_new(struct index_info,1);
+    arg->bindex = bindex;
     g_object_add_weak_pointer(G_OBJECT(bindex), (gpointer) &arg->bindex);
+    g_object_set_data(G_OBJECT(bindex), BNDX_IDLE_ARG, arg);
     g_idle_add((GSourceFunc) bndx_mailbox_changed_idle, arg);
 }
 
@@ -1686,7 +1754,7 @@ balsa_find_notebook_page_num(LibBalsaMailbox * mailbox)
     gint i;
 
     if (!balsa_app.notebook)
-	return -1;
+        return -1;
 
     for (i = 0;
          (page =
@@ -1694,7 +1762,8 @@ balsa_find_notebook_page_num(LibBalsaMailbox * mailbox)
          i++) {
         GtkWidget *index = gtk_bin_get_child(GTK_BIN(page));
 
-        if (BALSA_INDEX(index)->mailbox_node->mailbox == mailbox)
+        if (index && BALSA_INDEX(index)->mailbox_node
+            && BALSA_INDEX(index)->mailbox_node->mailbox == mailbox)
             return i;
     }
 
@@ -1995,55 +2064,13 @@ balsa_index_update_tree(BalsaIndex * index, gboolean expand)
      * deselected the current message */
     if (bndx_find_message(index, NULL, &iter, index->current_message))
         bndx_expand_to_row_and_select(index, &iter);
-    else {
-        /* Collapse can leave the display empty: scroll to one of the
-         * messages that was previously visible. */
-        guint msgno = GPOINTER_TO_UINT(g_object_get_data
-                                       (G_OBJECT(index),
-                                        "balsa-index-scroll-msgno"));
-        GtkTreePath *path;
-        if (msgno 
-            && libbalsa_mailbox_msgno_find(index->mailbox_node->mailbox,
-                                           msgno, &path, NULL)) {
-            bndx_scroll_to_row(index, path);
-            gtk_tree_path_free(path);
-        }
-    }
-    g_object_set_data(G_OBJECT(index), "balsa-index-scroll-msgno", NULL);
+    else
+        balsa_index_ensure_visible(index);
 
     bndx_changed_find_row(index);
 }
 
 /* balsa_index_set_threading_type: public method. */
-static void
-bndx_save_scroll_path(BalsaIndex * index)
-{
-    GtkTreeView *tree_view = GTK_TREE_VIEW(index);
-    GdkRectangle rect;
-    GtkTreePath *path;
-
-    gtk_tree_view_get_visible_rect(tree_view, &rect);
-    rect.x += rect.width/2;
-    rect.y += rect.height/2;
-    gtk_tree_view_tree_to_widget_coords(tree_view, rect.x, rect.y,
-                                        &rect.x, &rect.y);
-
-    if (gtk_tree_view_get_path_at_pos(tree_view, rect.x, rect.y, &path,
-                                      NULL, NULL, NULL)) {
-        guint msgno;
-
-        GtkTreeModel *model = gtk_tree_view_get_model(tree_view);
-        GtkTreeIter iter;
-
-        gtk_tree_model_get_iter(model, &iter, path);
-        gtk_tree_path_free(path);
-
-        gtk_tree_model_get(model, &iter, LB_MBOX_MSGNO_COL, &msgno, -1);
-        g_object_set_data(G_OBJECT(index), "balsa-index-scroll-msgno",
-                          GUINT_TO_POINTER(msgno));
-    }
-}
-
 void
 balsa_index_set_threading_type(BalsaIndex * index, int thtype)
 {
@@ -2055,9 +2082,6 @@ balsa_index_set_threading_type(BalsaIndex * index, int thtype)
     mailbox = index->mailbox_node->mailbox;
     g_return_if_fail(mailbox != NULL);
 
-    /* Save a message to scroll back to. */
-    bndx_save_scroll_path(index);
-
     if (thtype != LB_MAILBOX_THREADING_FLAT) 
         libbalsa_mailbox_prepare_threading(mailbox, NULL, 0);
     libbalsa_mailbox_set_threading_type(mailbox, thtype);
@@ -2070,11 +2094,11 @@ balsa_index_set_threading_type(BalsaIndex * index, int thtype)
 }
 
 void
-balsa_index_set_view_filter(BalsaIndex *bindex, int filter_no,
-                            const gchar *filter_string,
-                            LibBalsaCondition *filter)
+balsa_index_set_view_filter(BalsaIndex * bindex, int filter_no,
+                            const gchar * filter_string,
+                            LibBalsaCondition * filter)
 {
-    LibBalsaMailbox * mailbox;
+    LibBalsaMailbox *mailbox;
 
     g_return_if_fail(BALSA_IS_INDEX(bindex));
     mailbox = bindex->mailbox_node->mailbox;
@@ -2082,7 +2106,8 @@ balsa_index_set_view_filter(BalsaIndex *bindex, int filter_no,
     g_free(bindex->filter_string);
     bindex->filter_no = filter_no;
     bindex->filter_string = g_strdup(filter_string);
-    libbalsa_mailbox_set_view_filter(mailbox, filter, TRUE);
+    if (libbalsa_mailbox_set_view_filter(mailbox, filter, TRUE))
+        balsa_index_ensure_visible(bindex);
 }
 
 /* Public method. */
@@ -2116,11 +2141,12 @@ balsa_index_transfer(BalsaIndex *index, GArray * msgnos,
         libbalsa_mailbox_messages_move(from_mailbox, msgnos, to_mailbox, &e);
 
     if (!success) {
-	balsa_information(LIBBALSA_INFORMATION_WARNING,
-			  msgnos->len > 1 
-			  ? _("Failed to copy messages to mailbox \"%s\": %s")
-			  : _("Failed to copy message to mailbox \"%s\": %s"),
-			  to_mailbox->name, e ? e->message : "?");
+	balsa_information
+            (LIBBALSA_INFORMATION_WARNING,
+             ngettext("Failed to copy %d message to mailbox \"%s\": %s",
+                      "Failed to copy %d messages to mailbox \"%s\": %s",
+                      msgnos->len),
+             msgnos->len, to_mailbox->name, e ? e->message : "?");
 	return;
     }
 
@@ -2240,6 +2266,7 @@ balsa_index_expunge(BalsaIndex * index)
 {
     LibBalsaMailbox *mailbox;
     GtkTreeSelection *selection;
+    gboolean rc;
 
     g_return_if_fail(index != NULL);
 
@@ -2249,7 +2276,10 @@ balsa_index_expunge(BalsaIndex * index)
 
     selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(index));
     g_signal_handler_block(selection, index->selection_changed_id);
-    if (!libbalsa_mailbox_sync_storage(mailbox, TRUE))
+    gdk_threads_leave();
+    rc = libbalsa_mailbox_sync_storage(mailbox, TRUE);
+    gdk_threads_enter();
+    if (!rc)
 	balsa_information(LIBBALSA_INFORMATION_WARNING,
 			  _("Committing mailbox %s failed."),
 			  mailbox->name);
@@ -2388,7 +2418,7 @@ bndx_pipe_response(GtkWidget * dialog, gint response,
     g_object_set_data(G_OBJECT(mailbox), BALSA_INDEX_PIPE_INFO, NULL);
 }
 
-#define HIG_PADDING 6
+#define HIG_PADDING 12
 
 void
 balsa_index_pipe(BalsaIndex * index)
@@ -2431,10 +2461,10 @@ balsa_index_pipe(BalsaIndex * index)
                                     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                     NULL);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
+    gtk_container_set_border_width(GTK_CONTAINER(dialog), 5);
 
     vbox = GTK_DIALOG(dialog)->vbox;
     gtk_box_set_spacing(GTK_BOX(vbox), HIG_PADDING);
-    gtk_container_set_border_width(GTK_CONTAINER(vbox), HIG_PADDING);
     gtk_container_add(GTK_CONTAINER(vbox), label =
                       gtk_label_new(_("Specify the program to run:")));
 
@@ -2451,3 +2481,48 @@ balsa_index_pipe(BalsaIndex * index)
                      info);
     gtk_widget_show(dialog);
 }
+
+/* GtkTreeView can leave no messages showing after changing the view
+ * filter, even though the view does contain messages.  We scroll to
+ * either the current message, or the last message in the view, if
+ * any. */
+void
+balsa_index_ensure_visible(BalsaIndex * index)
+{
+    GtkTreeView *tree_view = GTK_TREE_VIEW(index);
+    GdkRectangle rect;
+    GtkTreePath *path = NULL;
+
+    gtk_tree_view_get_visible_rect(tree_view, &rect);
+    gtk_tree_view_tree_to_widget_coords(tree_view, rect.x, rect.y,
+                                        &rect.x, &rect.y);
+
+    if (gtk_tree_view_get_path_at_pos(tree_view, rect.x, rect.y, &path,
+                                      NULL, NULL, NULL)) {
+        /* We have a message in the view, so we do nothing. */
+        gtk_tree_path_free(path);
+        path = NULL;
+    } else {
+        /* Can we get a path to the current message? */
+        if (!bndx_find_message(index, &path, NULL, index->current_message)) {
+            /* No, scroll to the last message. */
+            GtkTreeModel *model;
+            gint n_children;
+
+            model = gtk_tree_view_get_model(tree_view);
+            n_children = gtk_tree_model_iter_n_children(model, NULL);
+
+            if (n_children > 0) {
+                GtkTreeIter iter;
+                gtk_tree_model_iter_nth_child(model, &iter, NULL,
+                                              --n_children);
+                path = gtk_tree_model_get_path(model, &iter);
+            }
+        }
+    }
+
+    if (path) {
+        bndx_scroll_to_row(index, path);
+        gtk_tree_path_free(path);
+    }
+}
