@@ -57,7 +57,6 @@ extern "C" {
         GtkWidget *move_to_item;
 
         BalsaMailboxNode* mailbox_node;
-        LibBalsaMessage* first_new_message;
         LibBalsaMessage* current_message;
 	gboolean current_message_is_deleted;
         gboolean prev_message;
@@ -119,7 +118,7 @@ extern "C" {
 
 /* select up/down the index */
     void balsa_index_select_next(BalsaIndex *);
-    void balsa_index_select_next_unread(BalsaIndex * bindex);
+    gboolean balsa_index_select_next_unread(BalsaIndex * index);
     void balsa_index_select_next_flagged(BalsaIndex * bindex);
     void balsa_index_select_previous(BalsaIndex *);
     void balsa_index_select(BalsaIndex * index, LibBalsaMessage * message);
@@ -167,6 +166,11 @@ extern "C" {
     /* Pipe messages */
     void balsa_index_pipe(BalsaIndex * index);
 
+    /* Make sure messages are visible. */
+    void balsa_index_ensure_visible(BalsaIndex * index);
+
+#define BALSA_INDEX_VIEW_ON_OPEN "balsa-index-view-on-open"
+
 #ifdef __cplusplus
 }
 #endif				/* __cplusplus */
