@@ -383,6 +383,8 @@ balsa_message_init(BalsaMessage * bm)
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                                    GTK_POLICY_AUTOMATIC,
                                    GTK_POLICY_AUTOMATIC);
+    g_signal_connect(scroll, "key_press_event",
+		     G_CALLBACK(balsa_mime_widget_key_press_event), bm);
     label = gtk_label_new(_("Content"));
     gtk_notebook_append_page(GTK_NOTEBOOK(bm), scroll, label);
     bm->cont_viewport = gtk_viewport_new(NULL, NULL);
@@ -523,6 +525,7 @@ balsa_message_set_close(BalsaMessage * bm, gboolean close_with_msg)
     bm->close_with_msg = close_with_msg;
 }
 
+/* Returns a BalsaPartInfo with a reference (g_object_unref when done). */
 static BalsaPartInfo *
 tree_next_valid_part_info(GtkTreeModel * model, GtkTreeIter * iter)
 {
@@ -533,10 +536,8 @@ tree_next_valid_part_info(GtkTreeModel * model, GtkTreeIter * iter)
 
         /* check if there is a valid info */
         gtk_tree_model_get(model, iter, PART_INFO_COLUMN, &info, -1);
-        if (info) {
-            g_object_unref(G_OBJECT(info));
+        if (info)
             return info;
-        }
 
         /* if there are children, check the childs */
         if (gtk_tree_model_iter_children (model, &child, iter))
@@ -566,8 +567,6 @@ tree_activate_row_cb(GtkTreeView *treeview, GtkTreePath *arg1,
     if (!gtk_tree_model_get_iter(model, &sel_iter, arg1))
         return;
     gtk_tree_model_get(model, &sel_iter, PART_INFO_COLUMN, &info, -1);
-    if (info)
-        g_object_unref(G_OBJECT(info));
     
     /* if it's not displayable (== no info), get the next one... */
     if (!info) {
@@ -576,15 +575,15 @@ tree_activate_row_cb(GtkTreeView *treeview, GtkTreePath *arg1,
         if (!info) {
             gtk_tree_model_get_iter_first(model, &sel_iter);
             gtk_tree_model_get(model, &sel_iter, PART_INFO_COLUMN, &info, -1);
-            if (info)
-                g_object_unref(G_OBJECT(info));
-            else
+            if (!info)
                 info = tree_next_valid_part_info(model, &sel_iter);
         }
     }
 
     gtk_notebook_set_current_page(GTK_NOTEBOOK(bm), 0);
     select_part(bm, info);
+    if (info)
+        g_object_unref(info);
 }
 
 static void
@@ -748,40 +747,6 @@ bm_message_weak_ref_cb(BalsaMessage * bm, LibBalsaMessage * message)
 
 /* Helpers:
  */
-static gboolean
-bm_content_has_focus(BalsaMessage * bm)
-{
-    GList *children, *list;
-    gboolean has_focus = FALSE;
-
-    children = gtk_container_get_children(GTK_CONTAINER(bm->bm_widget->container));
-    for (list = children; list; list = g_list_next(list)) {
-        GtkWidget *child = list->data;
-        if (GTK_WIDGET_HAS_FOCUS(child)) {
-            has_focus = TRUE;
-            break;
-        }
-    }
-    g_list_free(children);
-
-    return has_focus;
-}
-
-static void
-bm_focus_on_first_child(BalsaMessage * bm)
-{
-    GList *children, *list;
-
-    children = gtk_container_get_children(GTK_CONTAINER(bm->bm_widget->container));
-    for (list = children; list; list = g_list_next(list)) {
-        GtkWidget *child = list->data;
-        if (GTK_WIDGET_CAN_FOCUS(child)) {
-            gtk_widget_grab_focus(child);
-            break;
-        }
-    }
-    g_list_free(children);
-}
 
 gchar *
 balsa_message_sender_to_gchar(InternetAddressList * list, gint which)
@@ -813,9 +778,9 @@ gboolean
 balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
 {
     gboolean is_new;
-    gboolean has_focus;
     GtkTreeIter iter;
     BalsaPartInfo *info;
+    gboolean has_focus = bm->focus_state != BALSA_MESSAGE_FOCUS_STATE_NO;
 
     g_return_val_if_fail(bm != NULL, FALSE);
 
@@ -824,9 +789,6 @@ balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
     /*    if (bm->message == message) */
     /*      return; */
 
-    /* find out whether the content has the keyboard focus */
-    has_focus = bm_content_has_focus(bm);
-
     gtk_widget_hide(GTK_WIDGET(bm));
     select_part(bm, NULL);
     if (bm->message != NULL) {
@@ -900,25 +862,17 @@ balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
     if (is_new && message->headers->dispnotify_to)
         handle_mdn_request (message);
 
-    /*
-     * FIXME: This is a workaround for what may or may not be a libmutt bug.
-     *
-     * If the Content-Type: header is  multipart/alternative; boundary="XXX" 
-     * and no parts are found then mutt produces a message with no parts, even 
-     * if there is a single unmarked part (ie a normal email).     */
     if (!gtk_tree_model_get_iter_first (gtk_tree_view_get_model(GTK_TREE_VIEW(bm->treeview)),
-                                        &iter)) {
-        /* This is really annoying if you are browsing, since you keep
-           getting a dialog... */
-        /* balsa_information(LIBBALSA_INFORMATION_WARNING, _("Message
-           contains no parts!")); */
+                                        &iter))
+        /* Not possible? */
         return TRUE;
-    }
     
     info = 
         tree_next_valid_part_info(gtk_tree_view_get_model(GTK_TREE_VIEW(bm->treeview)),
                                   &iter);
     select_part(bm, info);
+    if (info)
+        g_object_unref(info);
     /*
      * emit read message
      */
@@ -932,7 +886,7 @@ balsa_message_set(BalsaMessage * bm, LibBalsaMessage * message)
 
     /* restore keyboard focus to the content, if it was there before */
     if (has_focus)
-        bm_focus_on_first_child(bm);
+        balsa_message_grab_focus(bm);
 
     return TRUE;
 }
@@ -1076,6 +1030,34 @@ add_to_attachments_popup(GtkMenuShell * menu, const gchar * item,
     gtk_menu_shell_append(menu, menuitem);
 }
 
+static void
+toggle_all_inline_cb(GtkCheckMenuItem * item, BalsaPartInfo *info)
+{
+    BalsaMessage * bm = g_object_get_data(G_OBJECT(item), "balsa-message");
+
+    g_return_if_fail(bm);
+    g_return_if_fail(info);
+    
+    bm->force_inline = gtk_check_menu_item_get_active(item);
+    
+    gtk_notebook_set_current_page(GTK_NOTEBOOK(bm), 0);
+    select_part(bm, info);
+}
+
+static void
+add_toggle_inline_menu_item(GtkMenuShell * menu, BalsaMessage * bm,
+			    BalsaPartInfo *info)
+{
+    GtkWidget * menuitem =
+	gtk_check_menu_item_new_with_label (_("force inline for all parts"));
+    
+    g_object_set_data(G_OBJECT(menuitem), "balsa-message", bm);
+    g_signal_connect(G_OBJECT (menuitem), "activate",
+		     GTK_SIGNAL_FUNC (toggle_all_inline_cb),
+		     (gpointer) info);
+    gtk_menu_shell_append(menu, menuitem);
+}
+
 static void
 display_part(BalsaMessage * bm, LibBalsaMessageBody * body,
              GtkTreeModel * model, GtkTreeIter * iter, gchar * part_id)
@@ -1106,6 +1088,10 @@ display_part(BalsaMessage * bm, LibBalsaMessageBody * body,
         } else if (is_multipart) {
             icon_title = mpart_content_name(content_type);
 	    if (!strcmp(part_id, "1")) {
+		add_toggle_inline_menu_item(GTK_MENU_SHELL(bm->parts_popup),
+					    bm, info);
+		gtk_menu_shell_append(GTK_MENU_SHELL(bm->parts_popup), 
+				      gtk_separator_menu_item_new ());
 		add_to_attachments_popup(GTK_MENU_SHELL(bm->parts_popup), 
 					 _("complete message"),
 					 bm, info);
@@ -1517,12 +1503,20 @@ balsa_message_next_part(BalsaMessage * bmessage)
     gtv = GTK_TREE_VIEW(bmessage->treeview);
     gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(gtv));
     select_part(bmessage, info);
+    g_object_unref(info);
 }
 
 gboolean
 balsa_message_has_next_part(BalsaMessage * bmessage)
 {
-    return bm_next_part_info(bmessage) != NULL;
+    BalsaPartInfo *info = bm_next_part_info(bmessage);
+
+    if (info) {
+        g_object_unref(info);
+        return TRUE;
+    }
+
+    return FALSE;
 }
 
 static BalsaPartInfo *
@@ -1563,7 +1557,6 @@ bm_previous_part_info(BalsaMessage * bmessage)
             gtk_tree_model_get(model, &sel.sel_iter, PART_INFO_COLUMN,
                                &info, -1);
         } while (!info);
-        g_object_unref(G_OBJECT(info));
         gtk_tree_path_free(path);
     }
             
@@ -1582,12 +1575,20 @@ balsa_message_previous_part(BalsaMessage * bmessage)
     gtv = GTK_TREE_VIEW(bmessage->treeview);
     gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(gtv));
     select_part(bmessage, info);
+    g_object_unref(info);
 }
 
 gboolean
 balsa_message_has_previous_part(BalsaMessage * bmessage)
 {
-    return bm_previous_part_info(bmessage) != NULL;
+    BalsaPartInfo *info = bm_previous_part_info(bmessage);
+
+    if (info) {
+        g_object_unref(info);
+        return TRUE;
+    }
+
+    return FALSE;
 }
 
 static LibBalsaMessageBody*
@@ -1666,11 +1667,11 @@ treeSearch_Func(GtkTreeModel * model, GtkTreePath *path,
 
     gtk_tree_model_get(model, iter, PART_INFO_COLUMN, &info, -1);
     if (info) {
-        if (info->body == search->body)
+        if (info->body == search->body) {
             search->info = info;
-        g_object_unref(G_OBJECT(info));
-        if (search->info)
             return TRUE;
+        } else
+            g_object_unref(info);
     }
 
     return FALSE;    
@@ -1697,9 +1698,10 @@ add_body(BalsaMessage *bm, LibBalsaMessageBody *body)
     if(body) {
         BalsaPartInfo *info = part_info_from_body(bm, body);
         
-        if (info)
+        if (info) {
 	    body = add_part(bm, info);
-        else
+            g_object_unref(info);
+        } else
 	    body = add_multipart(bm, body);
     }
 
@@ -1732,6 +1734,7 @@ add_multipart_mixed(BalsaMessage * bm, LibBalsaMessageBody * body)
 		g_mime_content_type_new_from_string(body->content_type);
 
             if (libbalsa_message_body_is_inline(body) ||
+		bm->force_inline ||
                 libbalsa_message_body_is_multipart(body) ||
 		g_mime_content_type_is_type(type, "application", "pgp-signature") ||
 		(balsa_app.has_smime && 
@@ -1740,8 +1743,9 @@ add_multipart_mixed(BalsaMessage * bm, LibBalsaMessageBody * body)
                 add_body(bm, body);
 	    g_mime_content_type_destroy(type);
 #else
-            if (libbalsa_message_body_is_inline(body)
-                || libbalsa_message_body_is_multipart(body))
+            if (libbalsa_message_body_is_inline(body) ||
+		bm->force_inline ||
+		libbalsa_message_body_is_multipart(body)) 
                 add_body(bm, body);
 #endif
         }
@@ -1856,6 +1860,7 @@ hide_all_parts(BalsaMessage * bm)
 			       gtk_tree_hide_func, bm);
 	gtk_tree_selection_unselect_all(gtk_tree_view_get_selection
 					(GTK_TREE_VIEW(bm->treeview)));
+        g_object_unref(bm->current_part);
 	bm->current_part = NULL;
     }
 
@@ -1872,6 +1877,9 @@ select_part(BalsaMessage * bm, BalsaPartInfo *info)
 {
     hide_all_parts(bm);
 
+    if (bm->current_part)
+        g_object_unref(bm->current_part);
+
     bm->current_part = part_info_from_body(bm, add_part(bm, info));
 
     if(bm->current_part)
@@ -1936,15 +1944,14 @@ balsa_message_can_select(BalsaMessage * bmessage)
 gboolean
 balsa_message_grab_focus(BalsaMessage * bmessage)
 {
-    GtkWidget *w;
-
     g_return_val_if_fail(bmessage != NULL, FALSE);
     g_return_val_if_fail(bmessage->current_part != NULL, FALSE);
-    g_return_val_if_fail((w =
-                          bmessage->current_part->mime_widget->widget) != NULL,
+    g_return_val_if_fail(bmessage->current_part->mime_widget->widget != NULL,
                          FALSE);
 
-    gtk_widget_grab_focus(w);
+    GTK_WIDGET_FLAGS(bmessage->current_part->mime_widget->widget)
+        |= GTK_CAN_FOCUS;
+    gtk_widget_grab_focus(bmessage->current_part->mime_widget->widget);
     return TRUE;
 }
 
@@ -2115,8 +2122,9 @@ static LibBalsaMessage *create_mdn_reply (LibBalsaMessage *for_msg,
 			       original_rcpt);	
     g_string_append_printf(report, "Final-Recipient: rfc822; %s\n",
 			   balsa_app.current_ident->ia->value.addr);
-    g_string_append_printf(report, "Original-Message-ID: %s\n",
-			   for_msg->message_id);
+    if (for_msg->message_id)
+        g_string_append_printf(report, "Original-Message-ID: <%s>\n",
+                               for_msg->message_id);
     g_string_append_printf(report,
 			   "Disposition: %s-action/MDN-sent-%sly; displayed",
 			   manual ? "manual" : "automatic",
@@ -2741,16 +2749,13 @@ message_recheck_crypto_cb(GtkWidget * button, BalsaMessage * bm)
     LibBalsaMessage * message;
     GtkTreeIter iter;
     BalsaPartInfo * info;
-    gboolean has_focus;
+    gboolean has_focus = bm->focus_state != BALSA_MESSAGE_FOCUS_STATE_NO;
 
     g_return_if_fail(bm != NULL);
 
     message = bm->message;
     g_return_if_fail(message != NULL);
 
-    /* find out whether the content has the keyboard focus */
-    has_focus = bm_content_has_focus(bm);
-
     select_part(bm, NULL);
     balsa_message_clear_tree(bm);
 
@@ -2797,10 +2802,12 @@ message_recheck_crypto_cb(GtkWidget * button, BalsaMessage * bm)
         tree_next_valid_part_info(gtk_tree_view_get_model(GTK_TREE_VIEW(bm->treeview)),
                                   &iter);
     select_part(bm, info);
+    if (info)
+        g_object_unref(info);
 
     /* restore keyboard focus to the content, if it was there before */
     if (has_focus)
-        bm_focus_on_first_child(bm);
+        balsa_message_grab_focus(bm);
 
     libbalsa_message_body_unref(message);
     g_object_unref(G_OBJECT(message));
