@@ -57,6 +57,7 @@ typedef enum {
     MBNODE_STYLE_NEW_MAIL = 1 << 1,
     MBNODE_STYLE_UNREAD_MESSAGES = 1 << 2,
     MBNODE_STYLE_TOTAL_MESSAGES = 1 << 3,
+    MBNODE_STYLE_UNREAD_CHILD = 1 << 4
 } BalsaMailboxNodeStyle;
 
 struct _BalsaMailboxNode {
@@ -64,18 +65,22 @@ struct _BalsaMailboxNode {
     BalsaMailboxNode *parent; /* NULL for root-level folders & mailboxes */
     LibBalsaMailbox *mailbox; /* != NULL for leaves only */
     gchar *name;       /* used for folders, i.e. when mailbox == NULL */
+    time_t last_use;   /* for closing least recently used mailboxes */
     BalsaMailboxNodeStyle style;
-    int remote:1;   /* is dirname or server field used in data union.
-		     * If there is a need for more types, make a subclass. */
     /* folder data */
     gchar* config_prefix;
     gchar* dir;      
-    int expanded:1; 
-    /* mailbox data */
     LibBalsaServer * server; /* Used only by remote; is referenced */
-    gboolean subscribed;     /* Used only by remote */
-    gboolean list_inbox;     /* Used only by remote */
-    gboolean scanned;        /* IMAP flag */
+    char delim; /* IMAP delimiter so that we do not need to check it
+		 * too often. */
+
+
+    unsigned remote:1;/* is dirname or server field used in data union.
+		       * If there is a need for more types, make a subclass. */
+
+    unsigned subscribed:1;     /* Used only by remote */
+    unsigned list_inbox:1;     /* Used only by remote */
+    unsigned scanned:1;        /* IMAP flag */
 };
 
 struct _BalsaMailboxNodeClass {
@@ -83,7 +88,7 @@ struct _BalsaMailboxNodeClass {
     void (*save_config) (BalsaMailboxNode * mn, const gchar * prefix);
     void (*load_config) (BalsaMailboxNode * mn, const gchar * prefix);
     GtkWidget* (*show_prop_dialog) (BalsaMailboxNode * mn);
-    void (*append_subtree) (BalsaMailboxNode * mn, GNode* root);
+    void (*append_subtree) (BalsaMailboxNode * mn);
 };
 
 GType balsa_mailbox_node_get_type(void);
@@ -98,13 +103,13 @@ BalsaMailboxNode *balsa_mailbox_node_new_from_config(const gchar* prefix);
 
 GtkWidget *balsa_mailbox_node_get_context_menu(BalsaMailboxNode * mbnode);
 void balsa_mailbox_node_show_prop_dialog(BalsaMailboxNode * mbnode);
-void balsa_mailbox_node_append_subtree(BalsaMailboxNode * mbnode, GNode *r);
+void balsa_mailbox_node_append_subtree(BalsaMailboxNode * mbnode);
 void balsa_mailbox_node_load_config(BalsaMailboxNode* mn, const gchar* prefix);
 void balsa_mailbox_node_save_config(BalsaMailboxNode* mn, const gchar* prefix);
 void balsa_mailbox_node_show_prop_dialog_cb(GtkWidget * widget, gpointer data);
 
 /* applicable only to local mailboxes (mailbox collections) */
-void balsa_mailbox_local_rescan_parent(LibBalsaMailbox* mbx);
+void balsa_mailbox_local_append(LibBalsaMailbox* mbx);
 /* applicable only to folders (mailbox collections) */
 void balsa_mailbox_node_rescan(BalsaMailboxNode* mn);
 /* applicable to any mailbox node */
