@@ -80,9 +80,6 @@ GIOChannel *mail_thread_msg_receive;
 GIOChannel *send_thread_msg_send;
 GIOChannel *send_thread_msg_receive;
 
-/* Semaphore to prevent dual use of appbar progressbar */
-int updating_progressbar;
-
 static void threads_init(void);
 static void threads_destroy(void);
 #endif				/* BALSA_USE_THREADS */
@@ -204,9 +201,10 @@ balsa_handle_automation_options() {
 static void
 balsa_init(int argc, char **argv)
 {
-    poptContext context;
-    int opt;
+#ifndef GNOME_PARAM_GOPTION_CONTEXT
     static char *attachment = NULL;
+    int opt;
+    poptContext context;
     static struct poptOption options[] = {
 
 	{"checkmail", 'c', POPT_ARG_NONE,
@@ -254,7 +252,72 @@ balsa_init(int argc, char **argv)
                        GNOME_PARAM_APP_DATADIR, BALSA_STD_PREFIX "/share",
 		       GNOME_PARAM_HUMAN_READABLE_NAME, _("The Balsa E-Mail Client"),
                        NULL);
+#else /* USE GOption interface */
+    static gchar **remaining_args = NULL;
+    static gchar **attach_vect = NULL;
+    static GOptionEntry option_entries[] = {
+	{"checkmail", 'c', 0, G_OPTION_ARG_NONE,
+	 &(cmd_check_mail_on_startup),
+	 N_("Get new mail on startup"), NULL},
+	{"compose", 'm', 0, G_OPTION_ARG_STRING, &(opt_compose_email),
+	 N_("Compose a new email to EMAIL@ADDRESS"), "EMAIL@ADDRESS"},
+	{"attach", 'a', 0, G_OPTION_ARG_FILENAME_ARRAY, &(attach_vect),
+	 N_("Attach file at PATH"), "PATH"},
+	{"open-mailbox", 'o', 0, G_OPTION_ARG_STRING,
+         &(cmd_line_open_mailboxes),
+	 N_("Opens MAILBOXNAME"), N_("MAILBOXNAME")},
+	{"open-unread-mailbox", 'u', 0, G_OPTION_ARG_NONE,
+	 &(cmd_open_unread_mailbox),
+	 N_("Opens first unread mailbox"), NULL},
+	{"open-inbox", 'i', 0, G_OPTION_ARG_NONE,
+	 &(cmd_open_inbox),
+	 N_("Opens default Inbox on startup"), NULL},
+	{"get-stats", 's', 0, G_OPTION_ARG_NONE,
+	 &(cmd_get_stats),
+	 N_("Prints number unread and unsent messages"), NULL},
+	{"debug-pop", 'd', 0, G_OPTION_ARG_NONE, &PopDebug,
+	 N_("Debug POP3 connection"), NULL},
+	{"debug-imap", 'D', 0, G_OPTION_ARG_NONE, &ImapDebug,
+	 N_("Debug IMAP connection"), NULL},
+        /* last but not least a special option that collects filenames */
+        { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY,
+          &remaining_args,
+          "Special option that collects any remaining arguments for us" },
+        { NULL }
+    };
+    GOptionContext *option_context = g_option_context_new("balsa");
+    GnomeProgram *my_app;
+    g_option_context_add_main_entries(option_context, option_entries, NULL);
 
+    my_app = gnome_program_init(PACKAGE, VERSION,
+                                LIBGNOMEUI_MODULE, argc, argv,
+                                GNOME_PARAM_GOPTION_CONTEXT, option_context,
+                                GNOME_PARAM_APP_DATADIR,
+                                BALSA_STD_PREFIX "/share",
+                                GNOME_PARAM_NONE);
+
+    if (remaining_args != NULL) {
+        gint i, num_args;
+        
+        num_args = g_strv_length (remaining_args);
+        for (i = 0; i < num_args; ++i) {
+            /* process remaining_args[i] here */
+            /* we do nothing for now */
+        }
+        g_strfreev (remaining_args);
+        remaining_args = NULL;
+    }
+    if (attach_vect != NULL) {
+        gint i, num_args;
+        
+        num_args = g_strv_length (attach_vect);
+        for (i = 0; i < num_args; ++i) {
+            opt_attach_list = g_slist_append(opt_attach_list, attach_vect[i]);
+        }
+        g_free(attach_vect);
+        attach_vect = NULL;
+    }
+#endif /* OPTION HANDLING */
     balsa_handle_automation_options();  
     
 }
@@ -340,7 +403,6 @@ threads_init(void)
     pthread_mutexattr_destroy(&attr);
     pthread_mutex_init(&send_messages_lock, NULL);
     checking_mail = 0;
-    updating_progressbar = 0;
     if (pipe(mail_thread_pipes) < 0) {
 	g_log("BALSA Init", G_LOG_LEVEL_DEBUG,
 	      "Error opening pipes.\n");
@@ -381,7 +443,7 @@ initial_open_unread_mailboxes()
 {
     GList *i, *gl;
     gdk_threads_enter();
-    gl = balsa_mblist_find_all_unread_mboxes();
+    gl = balsa_mblist_find_all_unread_mboxes(NULL);
 
     if (gl) {
         for (i = g_list_first(gl); i; i = g_list_next(i)) {
@@ -484,19 +546,14 @@ scan_mailboxes_idle_cb()
  * principle grow indefinetely. */
 static gboolean
 mbnode_expunge_func(GtkTreeModel *model, GtkTreePath *path,
-                    GtkTreeIter *iter)
+                    GtkTreeIter *iter, GSList ** list)
 {
     BalsaMailboxNode *mbnode;
 
     gtk_tree_model_get(model, iter, 0, &mbnode, -1);
     g_return_val_if_fail(mbnode, FALSE);
 
-    if (mbnode->mailbox && libbalsa_mailbox_is_open(mbnode->mailbox)) {
-        time_t tm = time(NULL);
-        if(tm-mbnode->last_use > balsa_app.expunge_timeout)
-            libbalsa_mailbox_sync_storage(mbnode->mailbox, TRUE);
-    }
-    g_object_unref(mbnode);
+    *list = g_slist_prepend(*list, mbnode);
 
     return FALSE;
 }
@@ -504,24 +561,81 @@ mbnode_expunge_func(GtkTreeModel *model, GtkTreePath *path,
 static gboolean
 periodic_expunge_cb(void)
 {
+    GSList *list = NULL, *l;
+
 #if !defined(ENABLE_TOUCH_UI)
     /* should we enforce expunging now and then? Perhaps not... */
     if(!balsa_app.expunge_auto) return TRUE;
 #endif
 
+    gdk_threads_enter();
     libbalsa_information(LIBBALSA_INFORMATION_MESSAGE,
                          _("Compressing mail folders..."));
-    /* make sure it is shown before we continue */
-    gdk_threads_enter();
     gtk_tree_model_foreach(GTK_TREE_MODEL(balsa_app.mblist_tree_store),
 			   (GtkTreeModelForeachFunc)mbnode_expunge_func,
-			   NULL);
+			   &list);
     gdk_threads_leave();
+
+    for (l = list; l; l = l->next) {
+        BalsaMailboxNode *mbnode = l->data;
+        if (mbnode->mailbox && libbalsa_mailbox_is_open(mbnode->mailbox)) {
+            time_t tm = time(NULL);
+            if (tm-mbnode->last_use > balsa_app.expunge_timeout)
+                libbalsa_mailbox_sync_storage(mbnode->mailbox, TRUE);
+        }
+        g_object_unref(mbnode);
+    }
+    g_slist_free(list);
+
     /* purge imap cache? leave 15MB */
     libbalsa_imap_purge_temp_dir(15*1024*1024);
+
     return TRUE; /* do it later as well */
 }
 
+/*
+ * Wrappers for libbalsa access to the progress bar.
+ */
+
+/*
+ * Initialize the progress bar and set text.
+ */
+static gdouble prev_fraction;
+static void
+balsa_progress_set_text(LibBalsaProgress * progress, const gchar * text,
+                        guint total)
+{
+    gboolean rc = FALSE;
+
+    gdk_threads_enter();
+    if (!text || total >= LIBBALSA_PROGRESS_MIN_COUNT)
+        rc = balsa_window_setup_progress(balsa_app.main_window, text);
+    prev_fraction = 0;
+    gdk_threads_leave();
+
+    *progress = (text && rc) ?
+        LIBBALSA_PROGRESS_YES : LIBBALSA_PROGRESS_NO;
+}
+
+/* 
+ * Set the fraction in the progress bar.
+ */
+
+static void
+balsa_progress_set_fraction(LibBalsaProgress * progress, gdouble fraction)
+{
+    if (*progress == LIBBALSA_PROGRESS_NO
+        || (fraction != 0 && fraction != 1
+            && fraction < prev_fraction + LIBBALSA_PROGRESS_MIN_UPDATE))
+        return;
+    prev_fraction = fraction;
+
+    gdk_threads_enter();
+    balsa_window_increment_progress(balsa_app.main_window, fraction,
+                                    !libbalsa_am_i_subthread());
+    gdk_threads_leave();
+}
+
 /* -------------------------- main --------------------------------- */
 int
 main(int argc, char *argv[])
@@ -572,6 +686,9 @@ main(int argc, char *argv[])
     libbalsa_init((LibBalsaInformationFunc) balsa_information_real);
     libbalsa_filters_set_url_mapper(balsa_find_mailbox_by_url);
     libbalsa_filters_set_filter_list(&balsa_app.filters);
+
+    libbalsa_progress_set_text     = balsa_progress_set_text;
+    libbalsa_progress_set_fraction = balsa_progress_set_fraction;
     
     /* checking for valid config files */
     config_init(cmd_get_stats);
