@@ -26,8 +26,7 @@
 extern "C" {
 #endif				/* __cplusplus */
 
-    void message_print_cb(GtkWidget * widget, gpointer cbdata);
-    void message_print(LibBalsaMessage * msg);
+    void message_print(LibBalsaMessage * msg, GtkWindow * parent);
 
 #ifdef __cplusplus
 }
