@@ -65,6 +65,9 @@ extern "C" {
 	GtkWidget *text;
 #if !HAVE_GTKSPELL
 	GtkWidget *spell_checker;
+#else
+	GtkWidget *spell_check_menu_item;
+        gboolean   spell_check_error;
 #endif                          /* HAVE_GTKSPELL */
 	GtkWidget *notebook;
 	LibBalsaMessage *parent_message; /* to which we're replying     */
@@ -73,7 +76,7 @@ extern "C" {
 	/* language selection related data */
 	gchar *charset;
 	GSList *charsets;
-	const gchar *locale;
+	const gchar *spell_check_lang;
 	GtkWidget *current_language_menu;
 	/* identity related data */
 	LibBalsaIdentity* ident;
@@ -87,13 +90,18 @@ extern "C" {
 	                        /* the config */
 	gulong delete_sig_id;
         gulong changed_sig_id;
+#if !HAVE_GTKSOURCEVIEW
         gulong delete_range_sig_id;
+#endif                          /* HAVE_GTKSOURCEVIEW */
         gulong insert_text_sig_id;
         guint wrap_timeout_id;
         guint autosave_timeout_id;
+	gboolean auto_saved;
+	gboolean user_saved;
         gulong identities_changed_id;
 	gboolean modified;
 	gboolean flow;          /* send format=flowed */ 
+	GtkWidget *flow_widget;
 	gboolean req_dispnotify; /* send a MDN */ 
 	gboolean quit_on_close; /* quit balsa after the compose window */
 	                        /* is closed.                          */
@@ -107,7 +115,9 @@ extern "C" {
 #endif
         GtkWidget *header_table;
 
+#if !HAVE_GTKSOURCEVIEW
         GtkTextBuffer *buffer2;       /* Undo buffer. */
+#endif                          /* HAVE_GTKSOURCEVIEW */
         GtkWidget *undo_widget;
         GtkWidget *redo_widget;
 
