@@ -68,7 +68,8 @@ extern "C" {
                                                GtkTextView * view);
     void balsa_spell_check_set_text(BalsaSpellCheck * spell_check,
 				    GtkTextView * view);
-    void balsa_spell_check_start(BalsaSpellCheck * spell_check);
+    void balsa_spell_check_start(BalsaSpellCheck * spell_check,
+                                 GtkWindow *parent);
 
 
 #ifdef __cplusplus
